@@ -190,6 +190,9 @@ static void gvBurnScanline(void *pCBData, int nY, int nXStart, int nXEnd,
         case GDT_Byte:
             gvBurnScanlineBasic<GByte>(psInfo, nY, nXStart, nXEnd, dfVariant);
             break;
+        case GDT_Int8:
+            gvBurnScanlineBasic<GInt8>(psInfo, nY, nXStart, nXEnd, dfVariant);
+            break;
         case GDT_Int16:
             gvBurnScanlineBasic<GInt16>(psInfo, nY, nXStart, nXEnd, dfVariant);
             break;
@@ -216,7 +219,12 @@ static void gvBurnScanline(void *pCBData, int nY, int nXStart, int nXEnd,
         case GDT_Float64:
             gvBurnScanlineBasic<double>(psInfo, nY, nXStart, nXEnd, dfVariant);
             break;
-        default:
+        case GDT_CInt16:
+        case GDT_CInt32:
+        case GDT_CFloat32:
+        case GDT_CFloat64:
+        case GDT_Unknown:
+        case GDT_TypeCount:
             CPLAssert(false);
             break;
     }
@@ -296,6 +304,9 @@ static void gvBurnPoint(void *pCBData, int nY, int nX, double dfVariant)
         case GDT_Byte:
             gvBurnPointBasic<GByte>(psInfo, nY, nX, dfVariant);
             break;
+        case GDT_Int8:
+            gvBurnPointBasic<GInt8>(psInfo, nY, nX, dfVariant);
+            break;
         case GDT_Int16:
             gvBurnPointBasic<GInt16>(psInfo, nY, nX, dfVariant);
             break;
@@ -320,7 +331,12 @@ static void gvBurnPoint(void *pCBData, int nY, int nX, double dfVariant)
         case GDT_Float64:
             gvBurnPointBasic<double>(psInfo, nY, nX, dfVariant);
             break;
-        default:
+        case GDT_CInt16:
+        case GDT_CInt32:
+        case GDT_CFloat32:
+        case GDT_CFloat64:
+        case GDT_Unknown:
+        case GDT_TypeCount:
             CPLAssert(false);
     }
 }
@@ -1164,8 +1180,8 @@ static CPLErr GDALRasterizeGeometriesInternal(
         for (int iShape = 0; iShape < nGeomCount; iShape++)
         {
 
-            OGRGeometry *poGeometry =
-                reinterpret_cast<OGRGeometry *>(pahGeometries[iShape]);
+            const OGRGeometry *poGeometry =
+                OGRGeometry::FromHandle(pahGeometries[iShape]);
             if (poGeometry == nullptr || poGeometry->IsEmpty())
                 continue;
             /* --------------------------------------------------------------------
@@ -1375,7 +1391,7 @@ CPLErr GDALRasterizeLayers(GDALDatasetH hDS, int nBandCount, int *panBandList,
     if (nBandCount == 0 || nLayerCount == 0)
         return CE_None;
 
-    GDALDataset *poDS = reinterpret_cast<GDALDataset *>(hDS);
+    GDALDataset *poDS = GDALDataset::FromHandle(hDS);
 
     // Prototype band.
     GDALRasterBand *poBand = poDS->GetRasterBand(panBandList[0]);
