@@ -150,6 +150,9 @@ static void DumpValue(CPLJSonStreamingWriter &serializer, const GByte *bytes,
         case GDT_Byte:
             DumpValue<GByte>(serializer, bytes);
             break;
+        case GDT_Int8:
+            DumpValue<GInt8>(serializer, bytes);
+            break;
         case GDT_Int16:
             DumpValue<GInt16>(serializer, bytes);
             break;
@@ -186,7 +189,8 @@ static void DumpValue(CPLJSonStreamingWriter &serializer, const GByte *bytes,
         case GDT_CFloat64:
             DumpComplexValue<double>(serializer, bytes);
             break;
-        default:
+        case GDT_Unknown:
+        case GDT_TypeCount:
             CPLAssert(false);
             break;
     }
