@@ -285,7 +285,7 @@ static void ReportOnLayer(OGRLayer *poLayer, int bVerbose)
             {
                 OGRGeomFieldDefn *poGFldDefn =
                     poLayer->GetLayerDefn()->GetGeomFieldDefn(iGeom);
-                OGRSpatialReference *poSRS = poGFldDefn->GetSpatialRef();
+                const OGRSpatialReference *poSRS = poGFldDefn->GetSpatialRef();
                 if (poSRS == nullptr)
                     pszWKT = CPLStrdup("(unknown)");
                 else
@@ -636,8 +636,11 @@ exit:
     if (poResultLayer != nullptr)
         poDS->ReleaseResultSet(poResultLayer);
 
-    if (poDS != nullptr)
-        GDALClose(poDS);
+    if (poDS)
+    {
+        if (GDALClose(poDS) != CE_None)
+            nRet = 1;
+    }
 
     GDALDestroyDriverManager();
 
