@@ -29,10 +29,9 @@
 
 #include "keamaskband.h"
 
-
 // constructor
-KEAMaskBand::KEAMaskBand(GDALRasterBand *pParent,
-                kealib::KEAImageIO *pImageIO, LockedRefCount *pRefCount)
+KEAMaskBand::KEAMaskBand(GDALRasterBand *pParent, kealib::KEAImageIO *pImageIO,
+                         LockedRefCount *pRefCount)
 {
     m_nSrcBand = pParent->GetBand();
     poDS = nullptr;
@@ -42,7 +41,7 @@ KEAMaskBand::KEAMaskBand(GDALRasterBand *pParent,
     nRasterYSize = pParent->GetYSize();
 
     eDataType = GDT_Byte;
-    pParent->GetBlockSize( &nBlockXSize, &nBlockYSize );
+    pParent->GetBlockSize(&nBlockXSize, &nBlockYSize);
     eAccess = pParent->GetAccess();
 
     // grab the imageio class and its refcount
@@ -58,7 +57,7 @@ KEAMaskBand::~KEAMaskBand()
     this->FlushCache(true);
 
     // decrement the recount and delete if needed
-    if( m_pRefCount->DecRef() )
+    if (m_pRefCount->DecRef())
     {
         try
         {
@@ -73,7 +72,7 @@ KEAMaskBand::~KEAMaskBand()
 }
 
 // overridden implementation - calls readImageBlock2BandMask instead
-CPLErr KEAMaskBand::IReadBlock( int nBlockXOff, int nBlockYOff, void * pImage )
+CPLErr KEAMaskBand::IReadBlock(int nBlockXOff, int nBlockYOff, void *pImage)
 {
     try
     {
@@ -81,33 +80,32 @@ CPLErr KEAMaskBand::IReadBlock( int nBlockXOff, int nBlockYOff, void * pImage )
         // we need to adjust the amount read so we don't go over the edge
         int nxsize = this->nBlockXSize;
         int nxtotalsize = this->nBlockXSize * (nBlockXOff + 1);
-        if( nxtotalsize > this->nRasterXSize )
+        if (nxtotalsize > this->nRasterXSize)
         {
             nxsize -= (nxtotalsize - this->nRasterXSize);
         }
         int nysize = this->nBlockYSize;
         int nytotalsize = this->nBlockYSize * (nBlockYOff + 1);
-        if( nytotalsize > this->nRasterYSize )
+        if (nytotalsize > this->nRasterYSize)
         {
             nysize -= (nytotalsize - this->nRasterYSize);
         }
-        this->m_pImageIO->readImageBlock2BandMask( this->m_nSrcBand,
-                                            pImage, this->nBlockXSize * nBlockXOff,
-                                            this->nBlockYSize * nBlockYOff,
-                                            nxsize, nysize, this->nBlockXSize, this->nBlockYSize,
-                                            kealib::kea_8uint );
+        this->m_pImageIO->readImageBlock2BandMask(
+            this->m_nSrcBand, pImage, this->nBlockXSize * nBlockXOff,
+            this->nBlockYSize * nBlockYOff, nxsize, nysize, this->nBlockXSize,
+            this->nBlockYSize, kealib::kea_8uint);
     }
     catch (kealib::KEAIOException &e)
     {
-        CPLError( CE_Failure, CPLE_AppDefined,
-                "Failed to read file: %s", e.what() );
+        CPLError(CE_Failure, CPLE_AppDefined, "Failed to read file: %s",
+                 e.what());
         return CE_Failure;
     }
     return CE_None;
 }
 
 // overridden implementation - calls writeImageBlock2BandMask instead
-CPLErr KEAMaskBand::IWriteBlock( int nBlockXOff, int nBlockYOff, void * pImage )
+CPLErr KEAMaskBand::IWriteBlock(int nBlockXOff, int nBlockYOff, void *pImage)
 {
     try
     {
@@ -115,27 +113,26 @@ CPLErr KEAMaskBand::IWriteBlock( int nBlockXOff, int nBlockYOff, void * pImage )
         // we need to adjust the amount written so we don't go over the edge
         int nxsize = this->nBlockXSize;
         int nxtotalsize = this->nBlockXSize * (nBlockXOff + 1);
-        if( nxtotalsize > this->nRasterXSize )
+        if (nxtotalsize > this->nRasterXSize)
         {
             nxsize -= (nxtotalsize - this->nRasterXSize);
         }
         int nysize = this->nBlockYSize;
         int nytotalsize = this->nBlockYSize * (nBlockYOff + 1);
-        if( nytotalsize > this->nRasterYSize )
+        if (nytotalsize > this->nRasterYSize)
         {
             nysize -= (nytotalsize - this->nRasterYSize);
         }
 
-        this->m_pImageIO-> writeImageBlock2BandMask( this->m_nSrcBand,
-                                            pImage, this->nBlockXSize * nBlockXOff,
-                                            this->nBlockYSize * nBlockYOff,
-                                            nxsize, nysize, this->nBlockXSize, this->nBlockYSize,
-                                            kealib::kea_8uint );
+        this->m_pImageIO->writeImageBlock2BandMask(
+            this->m_nSrcBand, pImage, this->nBlockXSize * nBlockXOff,
+            this->nBlockYSize * nBlockYOff, nxsize, nysize, this->nBlockXSize,
+            this->nBlockYSize, kealib::kea_8uint);
     }
     catch (kealib::KEAIOException &e)
     {
-        CPLError( CE_Failure, CPLE_AppDefined,
-                "Failed to write file: %s", e.what() );
+        CPLError(CE_Failure, CPLE_AppDefined, "Failed to write file: %s",
+                 e.what());
         return CE_Failure;
     }
     return CE_None;
