@@ -45,8 +45,8 @@
 #include <float.h>
 #include <map>
 
-//#define DEBUG_VERBOSE
-//#define DEBUG_QUERY
+// #define DEBUG_VERBOSE
+// #define DEBUG_QUERY
 
 #if defined(DEBUG_VERBOSE) && !defined(DEBUG_QUERY)
 #define DEBUG_QUERY
@@ -64,46 +64,47 @@
 /**
  * To move over the data return by queries
  **/
-#define POSTGIS_RASTER_VERSION          static_cast<GUInt16>(0)
-#define RASTER_HEADER_SIZE              61
-#define RASTER_BAND_HEADER_FIXED_SIZE   1
+#define POSTGIS_RASTER_VERSION static_cast<GUInt16>(0)
+#define RASTER_HEADER_SIZE 61
+#define RASTER_BAND_HEADER_FIXED_SIZE 1
 
-#define BAND_SIZE(nodatasize, datasize) \
-        (RASTER_BAND_HEADER_FIXED_SIZE + (nodatasize) + (datasize))
+#define BAND_SIZE(nodatasize, datasize)                                        \
+    (RASTER_BAND_HEADER_FIXED_SIZE + (nodatasize) + (datasize))
 
-#define GET_BAND_DATA(raster, nband, nodatasize, datasize) \
-    ((raster) + RASTER_HEADER_SIZE + (nband) * BAND_SIZE(nodatasize, datasize) - (datasize))
+#define GET_BAND_DATA(raster, nband, nodatasize, datasize)                     \
+    ((raster) + RASTER_HEADER_SIZE + (nband)*BAND_SIZE(nodatasize, datasize) - \
+     (datasize))
 
-#define GEOTRSFRM_TOPLEFT_X            0
-#define GEOTRSFRM_WE_RES               1
-#define GEOTRSFRM_ROTATION_PARAM1      2
-#define GEOTRSFRM_TOPLEFT_Y            3
-#define GEOTRSFRM_ROTATION_PARAM2      4
-#define GEOTRSFRM_NS_RES               5
+#define GEOTRSFRM_TOPLEFT_X 0
+#define GEOTRSFRM_WE_RES 1
+#define GEOTRSFRM_ROTATION_PARAM1 2
+#define GEOTRSFRM_TOPLEFT_Y 3
+#define GEOTRSFRM_ROTATION_PARAM2 4
+#define GEOTRSFRM_NS_RES 5
 
 // Number of results return by ST_Metadata PostGIS function
 #define ELEMENTS_OF_METADATA_RECORD 10
 
 // Positions of elements of ST_Metadata PostGIS function
-#define POS_UPPERLEFTX      0
-#define POS_UPPERLEFTY      1
-#define POS_WIDTH           2
-#define POS_HEIGHT          3
-#define POS_SCALEX          4
-#define POS_SCALEY          5
-#define POS_SKEWX           6
-#define POS_SKEWY           7
-#define POS_SRID            8
-#define POS_NBANDS          9
+#define POS_UPPERLEFTX 0
+#define POS_UPPERLEFTY 1
+#define POS_WIDTH 2
+#define POS_HEIGHT 3
+#define POS_SCALEX 4
+#define POS_SCALEY 5
+#define POS_SKEWX 6
+#define POS_SKEWY 7
+#define POS_SRID 8
+#define POS_NBANDS 9
 
 // Number of results return by ST_BandMetadata PostGIS function
 #define ELEMENTS_OF_BAND_METADATA_RECORD 4
 
 // Positions of elements of ST_BandMetadata PostGIS function
-#define POS_PIXELTYPE       0
-#define POS_NODATAVALUE     1
-#define POS_ISOUTDB         2
-#define POS_PATH            3
+#define POS_PIXELTYPE 0
+#define POS_NODATAVALUE 1
+#define POS_ISOUTDB 2
+#define POS_PATH 3
 
 /**
  * The driver can work in these modes:
@@ -131,7 +132,6 @@ typedef enum
     BROWSE_DATABASE
 } WorkingMode;
 
-
 enum class OutDBResolution
 {
     SERVER_SIDE,
@@ -142,28 +142,29 @@ enum class OutDBResolution
 /**
  * Important metadata of a PostGIS Raster band
  **/
-typedef struct {
+typedef struct
+{
     GDALDataType eDataType;
     int nBitsDepth;
-    GBool bSignedByte;
     GBool bHasNoDataValue;
     GBool bIsOffline;
-    char * path;
+    char *path;
     double dfNoDataValue;
 } BandMetadata;
 
-typedef struct {
-    char * pszSchema;
-    char * pszTable;
-    char * pszColumn;
+typedef struct
+{
+    char *pszSchema;
+    char *pszTable;
+    char *pszColumn;
     int nFactor;
 } PROverview;
 
 // Some tools definitions
-char * ReplaceQuotes(const char *, int);
-char * ReplaceSingleQuotes(const char *, int);
-char ** ParseConnectionString(const char *);
-GBool TranslateDataType(const char *, GDALDataType *, int *, GBool *);
+char *ReplaceQuotes(const char *, int);
+char *ReplaceSingleQuotes(const char *, int);
+char **ParseConnectionString(const char *);
+GBool TranslateDataType(const char *, GDALDataType *, int *);
 
 class PostGISRasterRasterBand;
 class PostGISRasterTileDataset;
@@ -172,31 +173,33 @@ class PostGISRasterTileDataset;
  * PostGISRasterDriver: extends GDALDriver to support PostGIS Raster
  * connect.
  **********************************************************************/
-class PostGISRasterDriver final: public GDALDriver {
+class PostGISRasterDriver final : public GDALDriver
+{
 
-private:
-    CPLMutex* hMutex;
-    std::map<CPLString, PGconn*> oMapConnection{};
+  private:
+    CPLMutex *hMutex;
+    std::map<CPLString, PGconn *> oMapConnection{};
 
     CPL_DISALLOW_COPY_ASSIGN(PostGISRasterDriver)
-public:
+  public:
     PostGISRasterDriver();
     virtual ~PostGISRasterDriver();
-  PGconn *GetConnection(const char *pszConnectionString,
-                        const char *pszServiceIn, const char *pszDbnameIn,
-                        const char *pszHostIn, const char *pszPortIn,
-                        const char *pszUserIn);
+    PGconn *GetConnection(const char *pszConnectionString,
+                          const char *pszServiceIn, const char *pszDbnameIn,
+                          const char *pszHostIn, const char *pszPortIn,
+                          const char *pszUserIn);
 };
 
 /***********************************************************************
  * PostGISRasterDataset: extends VRTDataset to support PostGIS Raster
  * datasets
  **********************************************************************/
-class PostGISRasterDataset final: public VRTDataset {
+class PostGISRasterDataset final : public VRTDataset
+{
     friend class PostGISRasterRasterBand;
     friend class PostGISRasterTileRasterBand;
-private:
 
+  private:
     typedef enum
     {
         LOWEST_RESOLUTION,
@@ -206,21 +209,21 @@ private:
         AVERAGE_APPROX_RESOLUTION
     } ResolutionStrategy;
 
-    char** papszSubdatasets;
+    char **papszSubdatasets;
     double adfGeoTransform[6];
     int nSrid;
     int nOverviewFactor;
     int nBandsToCreate;
-    PGconn* poConn;
+    PGconn *poConn;
     GBool bRegularBlocking;
     GBool bAllTilesSnapToSameGrid;
     GBool bCheckAllTiles;
-    char* pszSchema;
-    char* pszTable;
-    char* pszColumn;
-    char* pszWhere;
-    char * pszPrimaryKeyName;
-    GBool  bIsFastPK;
+    char *pszSchema;
+    char *pszTable;
+    char *pszColumn;
+    char *pszWhere;
+    char *pszPrimaryKeyName;
+    GBool bIsFastPK;
     int bHasTriedFetchingPrimaryKeyName;
     mutable OGRSpatialReference m_oSRS{};
     ResolutionStrategy resolutionStrategy;
@@ -232,15 +235,15 @@ private:
     double ymin;
     double xmax;
     double ymax;
-    PostGISRasterTileDataset ** papoSourcesHolders;
-    CPLQuadTree * hQuadTree;
+    PostGISRasterTileDataset **papoSourcesHolders;
+    CPLQuadTree *hQuadTree;
 
     GBool bHasBuiltOverviews;
     int nOverviewCount;
-    PostGISRasterDataset* poParentDS;
-    PostGISRasterDataset** papoOverviewDS;
+    PostGISRasterDataset *poParentDS;
+    PostGISRasterDataset **papoOverviewDS;
 
-    std::map<CPLString, PostGISRasterTileDataset*> oMapPKIDToRTDS{};
+    std::map<CPLString, PostGISRasterTileDataset *> oMapPKIDToRTDS{};
 
     GBool bAssumeMultiBandReadPattern;
     int nNextExpectedBand;
@@ -264,83 +267,79 @@ private:
     int m_nLastLoadSourcesYSize = 0;
     int m_nLastLoadSourcesBand = 0;
 
-    lru11::Cache<std::string, std::shared_ptr<GDALDataset>> oOutDBDatasetCache{8,0};
-    lru11::Cache<std::string, bool> oOutDBFilenameUsable{100,0};
+    lru11::Cache<std::string, std::shared_ptr<GDALDataset>> oOutDBDatasetCache{
+        8, 0};
+    lru11::Cache<std::string, bool> oOutDBFilenameUsable{100, 0};
 
     GBool ConstructOneDatasetFromTiles(PGresult *);
     GBool YieldSubdatasets(PGresult *, const char *);
     GBool SetRasterProperties(const char *);
     GBool BrowseDatabase(const char *, const char *);
-    GBool AddComplexSource(PostGISRasterTileDataset* poRTDS);
-    GBool GetDstWin(PostGISRasterTileDataset *, int *, int *, int *,
-                    int *);
-    BandMetadata * GetBandsMetadata(int *);
-    PROverview * GetOverviewTables(int *);
+    GBool AddComplexSource(PostGISRasterTileDataset *poRTDS);
+    void GetDstWin(PostGISRasterTileDataset *, int *, int *, int *, int *);
+    BandMetadata *GetBandsMetadata(int *);
+    PROverview *GetOverviewTables(int *);
 
-    PostGISRasterTileDataset* BuildRasterTileDataset(const char* pszMetadata,
-                                                     const char* pszPKID,
-                                                     int nBandsFetched,
-                                                     BandMetadata * poBandMetaData);
+    PostGISRasterTileDataset *
+    BuildRasterTileDataset(const char *pszMetadata, const char *pszPKID,
+                           int nBandsFetched, BandMetadata *poBandMetaData);
     void UpdateGlobalResolutionWithTileResolution(double tilePixelSizeX,
                                                   double tilePixelSizeY);
     void BuildOverviews();
-    void BuildBands(BandMetadata * poBandMetaData, int nBandsFetched);
+    void BuildBands(BandMetadata *poBandMetaData, int nBandsFetched);
 
-    PostGISRasterTileDataset * GetMatchingSourceRef(const char * pszPKID) { return oMapPKIDToRTDS[pszPKID]; }
-    PostGISRasterTileDataset * GetMatchingSourceRef(double dfUpperLeftX, double dfUpperLeftY);
+    PostGISRasterTileDataset *GetMatchingSourceRef(const char *pszPKID)
+    {
+        return oMapPKIDToRTDS[pszPKID];
+    }
+    PostGISRasterTileDataset *GetMatchingSourceRef(double dfUpperLeftX,
+                                                   double dfUpperLeftY);
 
-    bool CanUseClientSideOutDB(bool bAllBandCaching,
-                               int nBand,
-                               const CPLString& osWHERE);
+    bool CanUseClientSideOutDB(bool bAllBandCaching, int nBand,
+                               const CPLString &osWHERE);
 
-    bool LoadOutdbRaster(int& nCurOffset,
-                          GDALDataType eDT,
-                          int nBand,
-                          const GByte* pbyData,
-                          int nWKBLength,
-                          void* pImage,
-                          double dfTileUpperLeftX,
-                          double dfTileUpperLeftY,
-                          double dfTileResX,
-                          double dfTileResY,
-                          int nTileXSize,
-                          int nTileYSize);
+    bool LoadOutdbRaster(int &nCurOffset, GDALDataType eDT, int nBand,
+                         const GByte *pbyData, int nWKBLength, void *pImage,
+                         double dfTileUpperLeftX, double dfTileUpperLeftY,
+                         double dfTileResX, double dfTileResY, int nTileXSize,
+                         int nTileYSize);
 
     CPL_DISALLOW_COPY_ASSIGN(PostGISRasterDataset)
 
   protected:
-    virtual int         CloseDependentDatasets() override;
-    virtual void        FlushCache(bool bAtClosing) override;
+    virtual int CloseDependentDatasets() override;
+    virtual CPLErr FlushCache(bool bAtClosing) override;
 
-public:
+  public:
     PostGISRasterDataset();
     virtual ~PostGISRasterDataset();
-    static GDALDataset* Open(GDALOpenInfo *);
+    static GDALDataset *Open(GDALOpenInfo *);
     static int Identify(GDALOpenInfo *);
-    static GDALDataset* CreateCopy(const char *, GDALDataset *,
-        int, char **, GDALProgressFunc, void *);
-    static GBool InsertRaster(PGconn *, PostGISRasterDataset *,
-        const char *, const char *, const char *);
-    static CPLErr Delete(const char*);
-    virtual char      **GetMetadataDomainList() override;
-    char ** GetMetadata(const char *) override;
+    static GDALDataset *CreateCopy(const char *, GDALDataset *, int, char **,
+                                   GDALProgressFunc, void *);
+    static GBool InsertRaster(PGconn *, PostGISRasterDataset *, const char *,
+                              const char *, const char *);
+    static CPLErr Delete(const char *);
+    virtual char **GetMetadataDomainList() override;
+    char **GetMetadata(const char *) override;
 
-    const OGRSpatialReference* GetSpatialRef() const override;
-    CPLErr SetSpatialRef(const OGRSpatialReference* poSRS) override;
+    const OGRSpatialReference *GetSpatialRef() const override;
+    CPLErr SetSpatialRef(const OGRSpatialReference *poSRS) override;
 
     CPLErr SetGeoTransform(double *) override;
     CPLErr GetGeoTransform(double *) override;
     char **GetFileList() override;
 
-    int    GetOverviewCount();
-    PostGISRasterDataset* GetOverviewDS(int iOvr);
+    int GetOverviewCount();
+    PostGISRasterDataset *GetOverviewDS(int iOvr);
 
-    const char * GetPrimaryKeyRef();
+    const char *GetPrimaryKeyRef();
     GBool HasSpatialIndex();
     GBool LoadSources(int nXOff, int nYOff, int nXSize, int nYSize, int nBand);
-    GBool PolygonFromCoords(int nXOff, int nYOff, int nXEndOff,
-        int nYEndOff,double adfProjWin[8]);
-    void CacheTile(const char* pszMetadata, const char* pszRaster, const char *pszPKID, int nBand, bool bAllBandCaching);
+    GBool PolygonFromCoords(int nXOff, int nYOff, int nXEndOff, int nYEndOff,
+                            double adfProjWin[8]);
+    void CacheTile(const char *pszMetadata, const char *pszRaster,
+                   const char *pszPKID, int nBand, bool bAllBandCaching);
 };
 
 /***********************************************************************
@@ -349,43 +348,41 @@ public:
  **********************************************************************/
 class PostGISRasterTileRasterBand;
 
-class PostGISRasterRasterBand final: public VRTSourcedRasterBand {
+class PostGISRasterRasterBand final : public VRTSourcedRasterBand
+{
     friend class PostGISRasterDataset;
 
     CPL_DISALLOW_COPY_ASSIGN(PostGISRasterRasterBand)
-protected:
-    const char* pszSchema;
-    const char* pszTable;
-    const char* pszColumn;
+  protected:
+    const char *pszSchema;
+    const char *pszTable;
+    const char *pszColumn;
 
-    void                      NullBuffer(void* pData,
-                                         int nBufXSize,
-                                         int nBufYSize,
-                                         GDALDataType eBufType,
-                                         int nPixelSpace,
-                                         int nLineSpace);
-public:
+    void NullBuffer(void *pData, int nBufXSize, int nBufYSize,
+                    GDALDataType eBufType, int nPixelSpace, int nLineSpace);
 
-    PostGISRasterRasterBand( PostGISRasterDataset * poDSIn, int nBandIn,
-                             GDALDataType eDataTypeIn, GBool bNoDataValueSetIn,
-                             double dfNodata );
+  public:
+    PostGISRasterRasterBand(PostGISRasterDataset *poDSIn, int nBandIn,
+                            GDALDataType eDataTypeIn, GBool bNoDataValueSetIn,
+                            double dfNodata);
 
     virtual ~PostGISRasterRasterBand();
 
     virtual double GetNoDataValue(int *pbSuccess = nullptr) override;
     virtual CPLErr SetNoDataValue(double) override;
-    virtual CPLErr IRasterIO(GDALRWFlag, int, int, int, int, void *,
-                             int, int, GDALDataType,
-                             GSpacing nPixelSpace, GSpacing nLineSpace,
-                             GDALRasterIOExtraArg* psExtraArg) override;
+    virtual CPLErr IRasterIO(GDALRWFlag, int, int, int, int, void *, int, int,
+                             GDALDataType, GSpacing nPixelSpace,
+                             GSpacing nLineSpace,
+                             GDALRasterIOExtraArg *psExtraArg) override;
 
     virtual int GetOverviewCount() override;
-    virtual GDALRasterBand * GetOverview(int) override;
+    virtual GDALRasterBand *GetOverview(int) override;
     virtual GDALColorInterp GetColorInterpretation() override;
 
-    virtual double GetMinimum( int *pbSuccess ) override;
-    virtual double GetMaximum( int *pbSuccess ) override;
-    virtual CPLErr ComputeRasterMinMax( int bApproxOK, double* adfMinMax ) override;
+    virtual double GetMinimum(int *pbSuccess) override;
+    virtual double GetMaximum(int *pbSuccess) override;
+    virtual CPLErr ComputeRasterMinMax(int bApproxOK,
+                                       double *adfMinMax) override;
 };
 
 /***********************************************************************
@@ -393,47 +390,53 @@ public:
  **********************************************************************/
 class PostGISRasterTileRasterBand;
 
-class PostGISRasterTileDataset final: public GDALDataset {
+class PostGISRasterTileDataset final : public GDALDataset
+{
     friend class PostGISRasterDataset;
     friend class PostGISRasterRasterBand;
     friend class PostGISRasterTileRasterBand;
-private:
-    PostGISRasterDataset* poRDS;
-    char * pszPKID;
+
+  private:
+    PostGISRasterDataset *poRDS;
+    char *pszPKID;
     double adfGeoTransform[6];
 
     CPL_DISALLOW_COPY_ASSIGN(PostGISRasterTileDataset)
 
-public:
-    PostGISRasterTileDataset(PostGISRasterDataset* poRDS,
-                             int nXSize,
+  public:
+    PostGISRasterTileDataset(PostGISRasterDataset *poRDS, int nXSize,
                              int nYSize);
     ~PostGISRasterTileDataset();
     CPLErr GetGeoTransform(double *) override;
-    void   GetExtent(double* pdfMinX, double* pdfMinY, double* pdfMaxX, double* pdfMaxY) const;
-    const char* GetPKID() const { return pszPKID; }
+    void GetExtent(double *pdfMinX, double *pdfMinY, double *pdfMaxX,
+                   double *pdfMaxY) const;
+    const char *GetPKID() const
+    {
+        return pszPKID;
+    }
 };
 
 /***********************************************************************
  * PostGISRasterTileRasterBand: it holds a raster tile band, that will
  * be used as a source for PostGISRasterRasterBand
  **********************************************************************/
-class PostGISRasterTileRasterBand final: public GDALRasterBand {
+class PostGISRasterTileRasterBand final : public GDALRasterBand
+{
     friend class PostGISRasterRasterBand;
     friend class PostGISRasterDataset;
-private:
+
+  private:
     GBool IsCached();
 
-    VRTSource* poSource;
+    VRTSource *poSource;
 
     CPL_DISALLOW_COPY_ASSIGN(PostGISRasterTileRasterBand)
 
-public:
-    PostGISRasterTileRasterBand(
-        PostGISRasterTileDataset * poRTDS, int nBand,
-        GDALDataType eDataType);
+  public:
+    PostGISRasterTileRasterBand(PostGISRasterTileDataset *poRTDS, int nBand,
+                                GDALDataType eDataType);
     virtual ~PostGISRasterTileRasterBand();
     virtual CPLErr IReadBlock(int, int, void *) override;
 };
 
-#endif // POSTGISRASTER_H_INCLUDED
+#endif  // POSTGISRASTER_H_INCLUDED
