@@ -424,11 +424,12 @@ static bool LoadPythonAPI()
     if (libHandle == nullptr)
     {
         const char *const apszPythonSO[] = {
-            "libpython3.6m." SO_EXT, "libpython3.7m." SO_EXT,
             "libpython3.8." SO_EXT,  "libpython3.9." SO_EXT,
             "libpython3.10." SO_EXT, "libpython3.11." SO_EXT,
-            "libpython3.5m." SO_EXT, "libpython3.4m." SO_EXT,
-            "libpython3.3." SO_EXT,  "libpython3.2." SO_EXT};
+            "libpython3.12." SO_EXT, "libpython3.7m." SO_EXT,
+            "libpython3.6m." SO_EXT, "libpython3.5m." SO_EXT,
+            "libpython3.4m." SO_EXT, "libpython3.3." SO_EXT,
+            "libpython3.2." SO_EXT};
         for (size_t i = 0;
              libHandle == nullptr && i < CPL_ARRAYSIZE(apszPythonSO); ++i)
         {
@@ -626,9 +627,9 @@ static bool LoadPythonAPI()
     if (libHandle == nullptr)
     {
         const char *const apszPythonSO[] = {
-            "python36.dll",  "python37.dll",  "python38.dll", "python39.dll",
-            "python310.dll", "python311.dll", "python35.dll", "python34.dll",
-            "python33.dll",  "python32.dll"};
+            "python38.dll",  "python39.dll", "python310.dll", "python311.dll",
+            "python312.dll", "python37.dll", "python36.dll",  "python35.dll",
+            "python34.dll",  "python33.dll", "python32.dll"};
         UINT uOldErrorMode;
         uOldErrorMode =
             SetErrorMode(SEM_NOOPENFILEERRORBOX | SEM_FAILCRITICALERRORS);
@@ -636,7 +637,7 @@ static bool LoadPythonAPI()
         for (size_t i = 0;
              libHandle == nullptr && i < CPL_ARRAYSIZE(apszPythonSO); ++i)
         {
-            CPLDebug("GAL", "Trying %s", apszPythonSO[i]);
+            CPLDebug("GDAL", "Trying %s", apszPythonSO[i]);
             libHandle = LoadLibrary(apszPythonSO[i]);
             if (libHandle != nullptr)
                 CPLDebug("GDAL", "... success");
