@@ -702,7 +702,7 @@ void OGRCurvePolygon::setMeasured(OGRBoolean bIsMeasured)
 /*                       assignSpatialReference()                       */
 /************************************************************************/
 
-void OGRCurvePolygon::assignSpatialReference(OGRSpatialReference *poSR)
+void OGRCurvePolygon::assignSpatialReference(const OGRSpatialReference *poSR)
 {
     oCC.assignSpatialReference(this, poSR);
 }
