@@ -482,6 +482,9 @@ void RegisterOGRArrow()
         "Binary IntegerList Integer64List RealList StringList");
     poDriver->SetMetadataItem(GDAL_DMD_CREATIONFIELDDATASUBTYPES,
                               "Boolean Int16 Float32 JSON UUID");
+    poDriver->SetMetadataItem(GDAL_DMD_CREATION_FIELD_DEFN_FLAGS,
+                              "WidthPrecision Nullable "
+                              "Comment AlternativeName Domain");
 
     poDriver->pfnOpen = OGRFeatherDriverOpen;
     poDriver->pfnIdentify = OGRFeatherDriverIdentify;
