@@ -442,6 +442,24 @@ OGRErr OGREditableLayer::IUpsertFeature(OGRFeature *poFeature)
     }
 }
 
+/************************************************************************/
+/*                            IUpdateFeature()                          */
+/************************************************************************/
+
+OGRErr OGREditableLayer::IUpdateFeature(OGRFeature *poFeature,
+                                        int nUpdatedFieldsCount,
+                                        const int *panUpdatedFieldsIdx,
+                                        int nUpdatedGeomFieldsCount,
+                                        const int *panUpdatedGeomFieldsIdx,
+                                        bool bUpdateStyleString)
+{
+    // Do not use OGRLayerDecorator::IUpdateFeature() which will forward
+    // to the decorated layer
+    return OGRLayer::IUpdateFeature(
+        poFeature, nUpdatedFieldsCount, panUpdatedFieldsIdx,
+        nUpdatedGeomFieldsCount, panUpdatedGeomFieldsIdx, bUpdateStyleString);
+}
+
 /************************************************************************/
 /*                          DeleteFeature()                             */
 /************************************************************************/
@@ -788,6 +806,7 @@ OGRErr OGREditableLayer::AlterFieldDefn(int iField,
         poFieldDefn->SetNullable(poMemFieldDefn->IsNullable());
         poFieldDefn->SetUnique(poMemFieldDefn->IsUnique());
         poFieldDefn->SetDomainName(poMemFieldDefn->GetDomainName());
+        poFieldDefn->SetComment(poMemFieldDefn->GetComment());
         m_bStructureModified = true;
     }
     return eErr;
