@@ -29,7 +29,6 @@
 
 #include "ogrsf_frmts.h"
 
-
 /************************************************************************/
 /*                           OGRRegisterAll()                           */
 /************************************************************************/
@@ -242,7 +241,7 @@ void OGRRegisterAllInternal()
 #endif
 #ifdef NGW_ENABLED
     RegisterOGRNGW();
-#endif // NGW_ENABLED
+#endif  // NGW_ENABLED
 #ifdef MAPML_ENABLED
     RegisterOGRMapML();
 #endif
@@ -255,6 +254,9 @@ void OGRRegisterAllInternal()
 #ifdef ARROW_ENABLED
     RegisterOGRArrow();
 #endif
+#ifdef GTFS_ENABLED
+    RegisterOGRGTFS();
+#endif
 
     // NOTE: you need to generally insert your own driver before that line.
 
@@ -270,5 +272,4 @@ void OGRRegisterAllInternal()
     RegisterOGRAVCE00();
 #endif
 
-
 } /* OGRRegisterAll */
