@@ -260,7 +260,9 @@ class OGRGeoJSONDataSource final : public OGRDataSource
         return osJSonFlavor_;
     }
 
-    virtual void FlushCache(bool bAtClosing) override;
+    virtual CPLErr FlushCache(bool bAtClosing) override;
+
+    CPLErr Close() override;
 
     static const size_t SPACE_FOR_BBOX = 130;
 
@@ -293,7 +295,7 @@ class OGRGeoJSONDataSource final : public OGRDataSource
     //
     // Private utility functions
     //
-    void Clear();
+    bool Clear();
     int ReadFromFile(GDALOpenInfo *poOpenInfo, const char *pszUnprefixed);
     int ReadFromService(GDALOpenInfo *poOpenInfo, const char *pszSource);
     void LoadLayers(GDALOpenInfo *poOpenInfo, GeoJSONSourceType nSrcType,
