@@ -76,8 +76,28 @@ OGRGeoJSONDataSource::OGRGeoJSONDataSource()
 
 OGRGeoJSONDataSource::~OGRGeoJSONDataSource()
 {
-    OGRGeoJSONDataSource::FlushCache(true);
-    OGRGeoJSONDataSource::Clear();
+    OGRGeoJSONDataSource::Close();
+}
+
+/************************************************************************/
+/*                              Close()                                 */
+/************************************************************************/
+
+CPLErr OGRGeoJSONDataSource::Close()
+{
+    CPLErr eErr = CE_None;
+    if (nOpenFlags != OPEN_FLAGS_CLOSED)
+    {
+        if (OGRGeoJSONDataSource::FlushCache(true) != CE_None)
+            eErr = CE_Failure;
+
+        if (!OGRGeoJSONDataSource::Clear())
+            eErr = CE_Failure;
+
+        if (GDALDataset::Close() != CE_None)
+            eErr = CE_Failure;
+    }
+    return eErr;
 }
 
 /************************************************************************/
@@ -554,7 +574,7 @@ void OGRGeoJSONDataSource::SetAttributesTranslation(AttributesTranslation type)
 /*                  PRIVATE FUNCTIONS IMPLEMENTATION                    */
 /************************************************************************/
 
-void OGRGeoJSONDataSource::Clear()
+bool OGRGeoJSONDataSource::Clear()
 {
     for (int i = 0; i < nLayers_; i++)
     {
@@ -577,11 +597,14 @@ void OGRGeoJSONDataSource::Clear()
     pszGeoData_ = nullptr;
     nGeoDataLen_ = 0;
 
+    bool bRet = true;
     if (fpOut_)
     {
-        VSIFCloseL(fpOut_);
+        if (VSIFCloseL(fpOut_) != 0)
+            bRet = false;
         fpOut_ = nullptr;
     }
+    return bRet;
 }
 
 /************************************************************************/
@@ -1015,11 +1038,12 @@ void OGRGeoJSONDataSource::AddLayer(OGRGeoJSONLayer *poLayer)
 /*                            FlushCache()                              */
 /************************************************************************/
 
-void OGRGeoJSONDataSource::FlushCache(bool /*bAtClosing*/)
+CPLErr OGRGeoJSONDataSource::FlushCache(bool /*bAtClosing*/)
 {
     if (papoLayersWriter_ != nullptr)
-        return;
+        return CE_None;
 
+    CPLErr eErr = CE_None;
     for (int i = 0; i < nLayers_; i++)
     {
         if (papoLayers_[i]->HasBeenUpdated())
@@ -1113,9 +1137,10 @@ void OGRGeoJSONDataSource::FlushCache(bool /*bAtClosing*/)
                         }
                         else
                         {
-                            const bool bCopyOK = CPL_TO_BOOL(
+                            bool bCopyOK = CPL_TO_BOOL(
                                 VSIOverwriteFile(fpTarget, osNewFilename));
-                            VSIFCloseL(fpTarget);
+                            if (VSIFCloseL(fpTarget) != 0)
+                                bCopyOK = false;
                             if (bCopyOK)
                             {
                                 VSIUnlink(osNewFilename);
@@ -1150,10 +1175,13 @@ void OGRGeoJSONDataSource::FlushCache(bool /*bAtClosing*/)
                     }
                 }
             }
+            if (!bOK)
+                eErr = CE_Failure;
 
             // Restore filters.
             papoLayers_[i]->m_poAttrQuery = poAttrQueryBak;
             papoLayers_[i]->m_poFilterGeom = poFilterGeomBak;
         }
     }
+    return eErr;
 }
