@@ -38,10 +38,10 @@
 /************************************************************************/
 
 OGRParquetWriterLayer::OGRParquetWriterLayer(
-            arrow::MemoryPool* poMemoryPool,
-            const std::shared_ptr<arrow::io::OutputStream>& poOutputStream,
-            const char *pszLayerName):
-    OGRArrowWriterLayer(poMemoryPool, poOutputStream, pszLayerName)
+    arrow::MemoryPool *poMemoryPool,
+    const std::shared_ptr<arrow::io::OutputStream> &poOutputStream,
+    const char *pszLayerName)
+    : OGRArrowWriterLayer(poMemoryPool, poOutputStream, pszLayerName)
 {
 }
 
@@ -51,7 +51,7 @@ OGRParquetWriterLayer::OGRParquetWriterLayer(
 
 OGRParquetWriterLayer::~OGRParquetWriterLayer()
 {
-    if( m_bInitializationOK )
+    if (m_bInitializationOK)
         FinalizeWriting();
 }
 
@@ -59,16 +59,18 @@ OGRParquetWriterLayer::~OGRParquetWriterLayer()
 /*                       IsSupportedGeometryType()                      */
 /************************************************************************/
 
-bool OGRParquetWriterLayer::IsSupportedGeometryType(OGRwkbGeometryType eGType) const
+bool OGRParquetWriterLayer::IsSupportedGeometryType(
+    OGRwkbGeometryType eGType) const
 {
     const auto eFlattenType = wkbFlatten(eGType);
-    if( !OGR_GT_HasM(eGType) && eFlattenType <= wkbGeometryCollection )
+    if (!OGR_GT_HasM(eGType) && eFlattenType <= wkbGeometryCollection)
     {
         return true;
     }
 
-    const auto osConfigOptionName = "OGR_" + GetDriverUCName() + "_ALLOW_ALL_DIMS";
-    if( CPLTestBool(CPLGetConfigOption(osConfigOptionName.c_str(), "NO")) )
+    const auto osConfigOptionName =
+        "OGR_" + GetDriverUCName() + "_ALLOW_ALL_DIMS";
+    if (CPLTestBool(CPLGetConfigOption(osConfigOptionName.c_str(), "NO")))
     {
         return true;
     }
@@ -88,48 +90,49 @@ bool OGRParquetWriterLayer::SetOptions(CSLConstList papszOptions,
                                        OGRSpatialReference *poSpatialRef,
                                        OGRwkbGeometryType eGType)
 {
-    const char* pszGeomEncoding = CSLFetchNameValue(papszOptions, "GEOMETRY_ENCODING");
+    const char *pszGeomEncoding =
+        CSLFetchNameValue(papszOptions, "GEOMETRY_ENCODING");
     m_eGeomEncoding = OGRArrowGeomEncoding::WKB;
-    if( pszGeomEncoding )
+    if (pszGeomEncoding)
     {
-        if( EQUAL(pszGeomEncoding, "WKB") )
+        if (EQUAL(pszGeomEncoding, "WKB"))
             m_eGeomEncoding = OGRArrowGeomEncoding::WKB;
-        else if( EQUAL(pszGeomEncoding, "WKT") )
+        else if (EQUAL(pszGeomEncoding, "WKT"))
             m_eGeomEncoding = OGRArrowGeomEncoding::WKT;
-        else if( EQUAL(pszGeomEncoding, "GEOARROW") )
+        else if (EQUAL(pszGeomEncoding, "GEOARROW"))
             m_eGeomEncoding = OGRArrowGeomEncoding::GEOARROW_GENERIC;
         else
         {
             CPLError(CE_Failure, CPLE_NotSupported,
-                     "Unsupported GEOMETRY_ENCODING = %s",
-                     pszGeomEncoding);
+                     "Unsupported GEOMETRY_ENCODING = %s", pszGeomEncoding);
             return false;
         }
     }
 
     m_bForceCounterClockwiseOrientation =
-        EQUAL(CSLFetchNameValueDef(papszOptions, "POLYGON_ORIENTATION", "COUNTERCLOCKWISE"),
+        EQUAL(CSLFetchNameValueDef(papszOptions, "POLYGON_ORIENTATION",
+                                   "COUNTERCLOCKWISE"),
               "COUNTERCLOCKWISE");
 
-    if( eGType != wkbNone )
+    if (eGType != wkbNone)
     {
-        if( !IsSupportedGeometryType(eGType) )
+        if (!IsSupportedGeometryType(eGType))
         {
             return false;
         }
 
         m_poFeatureDefn->SetGeomType(eGType);
         auto eGeomEncoding = m_eGeomEncoding;
-        if( eGeomEncoding == OGRArrowGeomEncoding::GEOARROW_GENERIC )
+        if (eGeomEncoding == OGRArrowGeomEncoding::GEOARROW_GENERIC)
         {
             eGeomEncoding = GetPreciseArrowGeomEncoding(eGType);
-            if( eGeomEncoding == OGRArrowGeomEncoding::GEOARROW_GENERIC )
+            if (eGeomEncoding == OGRArrowGeomEncoding::GEOARROW_GENERIC)
                 return false;
         }
         m_aeGeomEncoding.push_back(eGeomEncoding);
         m_poFeatureDefn->GetGeomFieldDefn(0)->SetName(
             CSLFetchNameValueDef(papszOptions, "GEOMETRY_NAME", "geometry"));
-        if( poSpatialRef )
+        if (poSpatialRef)
         {
             auto poSRS = poSpatialRef->Clone();
             m_poFeatureDefn->GetGeomFieldDefn(0)->SetSpatialRef(poSRS);
@@ -139,12 +142,11 @@ bool OGRParquetWriterLayer::SetOptions(CSLConstList papszOptions,
 
     m_osFIDColumn = CSLFetchNameValueDef(papszOptions, "FID", "");
 
-    const char* pszCompression = CSLFetchNameValue(
-        papszOptions, "COMPRESSION");
-    if( pszCompression == nullptr )
+    const char *pszCompression = CSLFetchNameValue(papszOptions, "COMPRESSION");
+    if (pszCompression == nullptr)
     {
         auto oResult = arrow::util::Codec::GetCompressionType("snappy");
-        if( oResult.ok() && arrow::util::Codec::IsAvailable(*oResult) )
+        if (oResult.ok() && arrow::util::Codec::IsAvailable(*oResult))
         {
             pszCompression = "SNAPPY";
         }
@@ -154,50 +156,54 @@ bool OGRParquetWriterLayer::SetOptions(CSLConstList papszOptions,
         }
     }
 
-    if( EQUAL(pszCompression, "NONE") )
+    if (EQUAL(pszCompression, "NONE"))
         pszCompression = "UNCOMPRESSED";
     auto oResult = arrow::util::Codec::GetCompressionType(
-                        CPLString(pszCompression).tolower());
-    if( !oResult.ok() )
+        CPLString(pszCompression).tolower());
+    if (!oResult.ok())
     {
         CPLError(CE_Failure, CPLE_NotSupported,
                  "Unrecognized compression method: %s", pszCompression);
         return false;
     }
     m_eCompression = *oResult;
-    if( !arrow::util::Codec::IsAvailable(m_eCompression) )
+    if (!arrow::util::Codec::IsAvailable(m_eCompression))
     {
         CPLError(CE_Failure, CPLE_NotSupported,
                  "Compression method %s is known, but libarrow has not "
-                 "been built with support for it", pszCompression);
+                 "been built with support for it",
+                 pszCompression);
         return false;
     }
 
     m_oWriterPropertiesBuilder.compression(m_eCompression);
-    const std::string osCreator = CSLFetchNameValueDef(papszOptions, "CREATOR", "");
-    if( !osCreator.empty() )
+    const std::string osCreator =
+        CSLFetchNameValueDef(papszOptions, "CREATOR", "");
+    if (!osCreator.empty())
         m_oWriterPropertiesBuilder.created_by(osCreator);
     else
-        m_oWriterPropertiesBuilder.created_by("GDAL " GDAL_RELEASE_NAME ", using " CREATED_BY_VERSION);
+        m_oWriterPropertiesBuilder.created_by("GDAL " GDAL_RELEASE_NAME
+                                              ", using " CREATED_BY_VERSION);
 
     // Undocumented option. Not clear it is useful besides unit test purposes
-    if( !CPLTestBool(CSLFetchNameValueDef(papszOptions, "STATISTICS", "YES")) )
+    if (!CPLTestBool(CSLFetchNameValueDef(papszOptions, "STATISTICS", "YES")))
         m_oWriterPropertiesBuilder.disable_statistics();
 
-    if( m_eGeomEncoding == OGRArrowGeomEncoding::WKB && eGType != wkbNone )
+    if (m_eGeomEncoding == OGRArrowGeomEncoding::WKB && eGType != wkbNone)
     {
         m_oWriterPropertiesBuilder.disable_statistics(
             parquet::schema::ColumnPath::FromDotString(
                 m_poFeatureDefn->GetGeomFieldDefn(0)->GetNameRef()));
     }
 
-    const char* pszRowGroupSize = CSLFetchNameValue(papszOptions, "ROW_GROUP_SIZE");
-    if( pszRowGroupSize )
+    const char *pszRowGroupSize =
+        CSLFetchNameValue(papszOptions, "ROW_GROUP_SIZE");
+    if (pszRowGroupSize)
     {
         auto nRowGroupSize = static_cast<int64_t>(atoll(pszRowGroupSize));
-        if( nRowGroupSize > 0 )
+        if (nRowGroupSize > 0)
         {
-            if( nRowGroupSize > INT_MAX )
+            if (nRowGroupSize > INT_MAX)
                 nRowGroupSize = INT_MAX;
             m_nRowGroupSize = nRowGroupSize;
         }
@@ -217,7 +223,7 @@ bool OGRParquetWriterLayer::SetOptions(CSLConstList papszOptions,
 void OGRParquetWriterLayer::CloseFileWriter()
 {
     auto status = m_poFileWriter->Close();
-    if( !status.ok() )
+    if (!status.ok())
     {
         CPLError(CE_Failure, CPLE_AppDefined,
                  "FileWriter::Close() failed with %s",
@@ -229,44 +235,41 @@ void OGRParquetWriterLayer::CloseFileWriter()
 /*                            IdentifyCRS()                             */
 /************************************************************************/
 
-static OGRSpatialReference IdentifyCRS(const OGRSpatialReference* poSRS)
+static OGRSpatialReference IdentifyCRS(const OGRSpatialReference *poSRS)
 {
     OGRSpatialReference oSRSIdentified(*poSRS);
 
-    if( poSRS->GetAuthorityName(nullptr) == nullptr )
+    if (poSRS->GetAuthorityName(nullptr) == nullptr)
     {
         // Try to find a registered CRS that matches the input one
         int nEntries = 0;
-        int* panConfidence = nullptr;
-        OGRSpatialReferenceH* pahSRS =
+        int *panConfidence = nullptr;
+        OGRSpatialReferenceH *pahSRS =
             poSRS->FindMatches(nullptr, &nEntries, &panConfidence);
 
         // If there are several matches >= 90%, take the only one
         // that is EPSG
         int iOtherAuthority = -1;
         int iEPSG = -1;
-        const char* const apszOptions[] =
-        {
-            "IGNORE_DATA_AXIS_TO_SRS_AXIS_MAPPING=YES",
-            nullptr
-        };
+        const char *const apszOptions[] = {
+            "IGNORE_DATA_AXIS_TO_SRS_AXIS_MAPPING=YES", nullptr};
         int iConfidenceBestMatch = -1;
-        for(int iSRS = 0; iSRS < nEntries; iSRS++ )
+        for (int iSRS = 0; iSRS < nEntries; iSRS++)
         {
             auto poCandidateCRS = OGRSpatialReference::FromHandle(pahSRS[iSRS]);
-            if( panConfidence[iSRS] < iConfidenceBestMatch ||
-                panConfidence[iSRS] < 70 )
+            if (panConfidence[iSRS] < iConfidenceBestMatch ||
+                panConfidence[iSRS] < 70)
             {
                 break;
             }
-            if( poSRS->IsSame(poCandidateCRS, apszOptions) )
+            if (poSRS->IsSame(poCandidateCRS, apszOptions))
             {
-                const char* pszAuthName =
-                   poCandidateCRS->GetAuthorityName(nullptr);
-                if( pszAuthName != nullptr && EQUAL(pszAuthName, "EPSG") )
+                const char *pszAuthName =
+                    poCandidateCRS->GetAuthorityName(nullptr);
+                if (pszAuthName != nullptr && EQUAL(pszAuthName, "EPSG"))
                 {
                     iOtherAuthority = -2;
-                    if( iEPSG < 0 )
+                    if (iEPSG < 0)
                     {
                         iConfidenceBestMatch = panConfidence[iSRS];
                         iEPSG = iSRS;
@@ -277,18 +280,19 @@ static OGRSpatialReference IdentifyCRS(const OGRSpatialReference* poSRS)
                         break;
                     }
                 }
-                else if( iEPSG < 0 && pszAuthName != nullptr )
+                else if (iEPSG < 0 && pszAuthName != nullptr)
                 {
-                    if( EQUAL(pszAuthName, "OGC") )
+                    if (EQUAL(pszAuthName, "OGC"))
                     {
-                        const char* pszAuthCode = poCandidateCRS->GetAuthorityCode(nullptr);
-                        if( pszAuthCode && EQUAL(pszAuthCode, "CRS84") )
+                        const char *pszAuthCode =
+                            poCandidateCRS->GetAuthorityCode(nullptr);
+                        if (pszAuthCode && EQUAL(pszAuthCode, "CRS84"))
                         {
                             iOtherAuthority = iSRS;
                             break;
                         }
                     }
-                    else if( iOtherAuthority == -1 )
+                    else if (iOtherAuthority == -1)
                     {
                         iConfidenceBestMatch = panConfidence[iSRS];
                         iOtherAuthority = iSRS;
@@ -298,13 +302,14 @@ static OGRSpatialReference IdentifyCRS(const OGRSpatialReference* poSRS)
                 }
             }
         }
-        if( iEPSG >= 0 )
+        if (iEPSG >= 0)
         {
             oSRSIdentified = *OGRSpatialReference::FromHandle(pahSRS[iEPSG]);
         }
-        else if( iOtherAuthority >= 0 )
+        else if (iOtherAuthority >= 0)
         {
-            oSRSIdentified = *OGRSpatialReference::FromHandle(pahSRS[iOtherAuthority]);
+            oSRSIdentified =
+                *OGRSpatialReference::FromHandle(pahSRS[iOtherAuthority]);
         }
         OSRFreeSRSArray(pahSRS);
         CPLFree(panConfidence);
@@ -317,23 +322,23 @@ static OGRSpatialReference IdentifyCRS(const OGRSpatialReference* poSRS)
 /*                      RemoveIDFromMemberOfEnsembles()                 */
 /************************************************************************/
 
-static void RemoveIDFromMemberOfEnsembles(CPLJSONObject& obj)
+static void RemoveIDFromMemberOfEnsembles(CPLJSONObject &obj)
 {
     // Remove "id" from members of datum ensembles for compatibility with
     // older PROJ versions
     // Cf https://github.com/opengeospatial/geoparquet/discussions/110
     // and https://github.com/OSGeo/PROJ/pull/3221
-    if( obj.GetType() == CPLJSONObject::Type::Object )
+    if (obj.GetType() == CPLJSONObject::Type::Object)
     {
-        for( auto& subObj: obj.GetChildren() )
+        for (auto &subObj : obj.GetChildren())
         {
             RemoveIDFromMemberOfEnsembles(subObj);
         }
     }
-    else if( obj.GetType() == CPLJSONObject::Type::Array &&
-             obj.GetName() == "members" )
+    else if (obj.GetType() == CPLJSONObject::Type::Array &&
+             obj.GetName() == "members")
     {
-        for( auto& subObj: obj.ToArray() )
+        for (auto &subObj : obj.ToArray())
         {
             subObj.Delete("id");
         }
@@ -346,16 +351,16 @@ static void RemoveIDFromMemberOfEnsembles(CPLJSONObject& obj)
 
 std::string OGRParquetWriterLayer::GetGeoMetadata() const
 {
-    if( m_poFeatureDefn->GetGeomFieldCount() != 0 &&
-        CPLTestBool(CPLGetConfigOption("OGR_PARQUET_WRITE_GEO", "YES")) )
+    if (m_poFeatureDefn->GetGeomFieldCount() != 0 &&
+        CPLTestBool(CPLGetConfigOption("OGR_PARQUET_WRITE_GEO", "YES")))
     {
         CPLJSONObject oRoot;
-        oRoot.Add("version", "0.4.0");
+        oRoot.Add("version", "1.0.0-beta.1");
         oRoot.Add("primary_column",
                   m_poFeatureDefn->GetGeomFieldDefn(0)->GetNameRef());
         CPLJSONObject oColumns;
         oRoot.Add("columns", oColumns);
-        for( int i = 0; i < m_poFeatureDefn->GetGeomFieldCount(); ++i )
+        for (int i = 0; i < m_poFeatureDefn->GetGeomFieldCount(); ++i)
         {
             const auto poGeomFieldDefn = m_poFeatureDefn->GetGeomFieldDefn(i);
             CPLJSONObject oColumn;
@@ -363,39 +368,41 @@ std::string OGRParquetWriterLayer::GetGeoMetadata() const
             oColumn.Add("encoding",
                         GetGeomEncodingAsString(m_aeGeomEncoding[i]));
 
-            if( CPLTestBool(CPLGetConfigOption(
-                    "OGR_PARQUET_WRITE_CRS", "YES")) )
+            if (CPLTestBool(CPLGetConfigOption("OGR_PARQUET_WRITE_CRS", "YES")))
             {
                 const auto poSRS = poGeomFieldDefn->GetSpatialRef();
-                if( poSRS )
+                if (poSRS)
                 {
                     OGRSpatialReference oSRSIdentified(IdentifyCRS(poSRS));
 
-                    const char* pszAuthName =
+                    const char *pszAuthName =
                         oSRSIdentified.GetAuthorityName(nullptr);
-                    const char* pszAuthCode =
+                    const char *pszAuthCode =
                         oSRSIdentified.GetAuthorityCode(nullptr);
 
                     bool bOmitCRS = false;
-                    if( pszAuthName != nullptr && pszAuthCode != nullptr &&
-                        ((EQUAL(pszAuthName, "EPSG") && EQUAL(pszAuthCode, "4326")) ||
-                         (EQUAL(pszAuthName, "OGC") && EQUAL(pszAuthCode, "CRS84"))) )
+                    if (pszAuthName != nullptr && pszAuthCode != nullptr &&
+                        ((EQUAL(pszAuthName, "EPSG") &&
+                          EQUAL(pszAuthCode, "4326")) ||
+                         (EQUAL(pszAuthName, "OGC") &&
+                          EQUAL(pszAuthCode, "CRS84"))))
                     {
                         // To make things less confusing for non-geo-aware
                         // consumers, omit EPSG:4326 / OGC:CRS84 CRS by default
-                        bOmitCRS = CPLTestBool(
-                            CPLGetConfigOption("OGR_PARQUET_CRS_OMIT_IF_WGS84", "YES"));
+                        bOmitCRS = CPLTestBool(CPLGetConfigOption(
+                            "OGR_PARQUET_CRS_OMIT_IF_WGS84", "YES"));
                     }
 
-                    if( bOmitCRS )
+                    if (bOmitCRS)
                     {
                         // do nothing
                     }
-                    else if( EQUAL(CPLGetConfigOption(
-                        "OGR_PARQUET_CRS_ENCODING", "PROJJSON"), "PROJJSON") )
+                    else if (EQUAL(CPLGetConfigOption(
+                                       "OGR_PARQUET_CRS_ENCODING", "PROJJSON"),
+                                   "PROJJSON"))
                     {
                         // CRS encoded as PROJJSON for GeoParquet >= 0.4.0
-                        char* pszPROJJSON = nullptr;
+                        char *pszPROJJSON = nullptr;
                         oSRSIdentified.exportToPROJJSON(&pszPROJJSON, nullptr);
                         CPLJSONDocument oCRSDoc;
                         oCRSDoc.LoadMemory(pszPROJJSON);
@@ -407,17 +414,17 @@ std::string OGRParquetWriterLayer::GetGeoMetadata() const
                     else
                     {
                         // WKT was used in GeoParquet <= 0.3.0
-                        const char* const apszOptions[] = {
-                            "FORMAT=WKT2_2019", "MULTILINE=NO", nullptr };
-                        char* pszWKT = nullptr;
+                        const char *const apszOptions[] = {
+                            "FORMAT=WKT2_2019", "MULTILINE=NO", nullptr};
+                        char *pszWKT = nullptr;
                         oSRSIdentified.exportToWkt(&pszWKT, apszOptions);
-                        if( pszWKT )
+                        if (pszWKT)
                             oColumn.Add("crs", pszWKT);
                         CPLFree(pszWKT);
                     }
 
                     const double dfCoordEpoch = poSRS->GetCoordinateEpoch();
-                    if( dfCoordEpoch > 0 )
+                    if (dfCoordEpoch > 0)
                         oColumn.Add("epoch", dfCoordEpoch);
                 }
                 else
@@ -426,20 +433,31 @@ std::string OGRParquetWriterLayer::GetGeoMetadata() const
                 }
             }
 
-            if( m_bEdgesSpherical )
+            if (m_bEdgesSpherical)
             {
                 oColumn.Add("edges", "spherical");
             }
 
-            if( m_aoEnvelopes[i].IsInit() &&
-                CPLTestBool(CPLGetConfigOption(
-                    "OGR_PARQUET_WRITE_BBOX", "YES")) )
+            if (m_aoEnvelopes[i].IsInit() &&
+                CPLTestBool(
+                    CPLGetConfigOption("OGR_PARQUET_WRITE_BBOX", "YES")))
             {
+                bool bHasZ = false;
+                for (const auto eGeomType : m_oSetWrittenGeometryTypes[i])
+                {
+                    bHasZ = OGR_GT_HasZ(eGeomType);
+                    if (bHasZ)
+                        break;
+                }
                 CPLJSONArray oBBOX;
                 oBBOX.Add(m_aoEnvelopes[i].MinX);
                 oBBOX.Add(m_aoEnvelopes[i].MinY);
+                if (bHasZ)
+                    oBBOX.Add(m_aoEnvelopes[i].MinZ);
                 oBBOX.Add(m_aoEnvelopes[i].MaxX);
                 oBBOX.Add(m_aoEnvelopes[i].MaxY);
+                if (bHasZ)
+                    oBBOX.Add(m_aoEnvelopes[i].MaxZ);
                 oColumn.Add("bbox", oBBOX);
             }
 
@@ -447,56 +465,43 @@ std::string OGRParquetWriterLayer::GetGeoMetadata() const
             {
                 const auto eFlattenType = wkbFlatten(eType);
                 std::string osType = "Unknown";
-                if( wkbPoint == eFlattenType )
+                if (wkbPoint == eFlattenType)
                     osType = "Point";
-                else if( wkbLineString == eFlattenType )
+                else if (wkbLineString == eFlattenType)
                     osType = "LineString";
-                else if( wkbPolygon == eFlattenType )
+                else if (wkbPolygon == eFlattenType)
                     osType = "Polygon";
-                else if( wkbMultiPoint == eFlattenType )
+                else if (wkbMultiPoint == eFlattenType)
                     osType = "MultiPoint";
-                else if( wkbMultiLineString == eFlattenType )
+                else if (wkbMultiLineString == eFlattenType)
                     osType = "MultiLineString";
-                else if( wkbMultiPolygon == eFlattenType )
+                else if (wkbMultiPolygon == eFlattenType)
                     osType = "MultiPolygon";
-                else if( wkbGeometryCollection == eFlattenType )
+                else if (wkbGeometryCollection == eFlattenType)
                     osType = "GeometryCollection";
-                if( osType != "Unknown" )
+                if (osType != "Unknown")
                 {
                     // M and ZM not supported officially currently, but it
                     // doesn't hurt to anticipate
-                    if( OGR_GT_HasZ(eType) && OGR_GT_HasM(eType) )
+                    if (OGR_GT_HasZ(eType) && OGR_GT_HasM(eType))
                         osType += " ZM";
-                    else if( OGR_GT_HasZ(eType) )
+                    else if (OGR_GT_HasZ(eType))
                         osType += " Z";
-                    else if( OGR_GT_HasM(eType) )
+                    else if (OGR_GT_HasM(eType))
                         osType += " M";
                 }
                 return osType;
             };
 
-            if( m_bForceCounterClockwiseOrientation )
+            if (m_bForceCounterClockwiseOrientation)
                 oColumn.Add("orientation", "counterclockwise");
 
-            if( m_oSetWrittenGeometryTypes[i].empty() )
+            CPLJSONArray oArray;
+            for (const auto eType : m_oSetWrittenGeometryTypes[i])
             {
-                const auto eType = poGeomFieldDefn->GetType();
-                oColumn.Add("geometry_type", GetStringGeometryType(eType));
-            }
-            else if( m_oSetWrittenGeometryTypes[i].size() == 1 )
-            {
-                const auto eType = *(m_oSetWrittenGeometryTypes[i].begin());
-                oColumn.Add("geometry_type", GetStringGeometryType(eType));
-            }
-            else
-            {
-                CPLJSONArray oArray;
-                for( const auto eType: m_oSetWrittenGeometryTypes[i] )
-                {
-                    oArray.Add(GetStringGeometryType(eType));
-                }
-                oColumn.Add("geometry_type", oArray);
+                oArray.Add(GetStringGeometryType(eType));
             }
+            oColumn.Add("geometry_types", oArray);
         }
 
         return oRoot.Format(CPLJSONObject::PrettyFormat::Plain);
@@ -510,35 +515,41 @@ std::string OGRParquetWriterLayer::GetGeoMetadata() const
 
 void OGRParquetWriterLayer::PerformStepsBeforeFinalFlushGroup()
 {
-    if( m_poKeyValueMetadata )
+    if (m_poKeyValueMetadata)
     {
         const std::string osGeoMetadata = GetGeoMetadata();
         auto poTmpSchema = m_poSchema;
-        if( !osGeoMetadata.empty() )
+        if (!osGeoMetadata.empty())
         {
             // HACK: it would be good for Arrow to provide a clean way to alter
             // key value metadata before finalizing.
             // We need to write metadata at end to write the bounding box.
-            const_cast<arrow::KeyValueMetadata*>(m_poKeyValueMetadata.get())->Append(
-                    "geo", osGeoMetadata);
+            const_cast<arrow::KeyValueMetadata *>(m_poKeyValueMetadata.get())
+                ->Append("geo", osGeoMetadata);
 
-            auto kvMetadata = poTmpSchema->metadata() ? poTmpSchema->metadata()->Copy() :
-                                  std::make_shared<arrow::KeyValueMetadata>();
+            auto kvMetadata = poTmpSchema->metadata()
+                                  ? poTmpSchema->metadata()->Copy()
+                                  : std::make_shared<arrow::KeyValueMetadata>();
             kvMetadata->Append("geo", osGeoMetadata);
             poTmpSchema = poTmpSchema->WithMetadata(kvMetadata);
         }
 
-        if( CPLTestBool(CPLGetConfigOption("OGR_PARQUET_WRITE_ARROW_SCHEMA", "YES")) )
+        if (CPLTestBool(
+                CPLGetConfigOption("OGR_PARQUET_WRITE_ARROW_SCHEMA", "YES")))
         {
-            auto status = ::arrow::ipc::SerializeSchema(*poTmpSchema, m_poMemoryPool);
-            if( status.ok() )
+            auto status =
+                ::arrow::ipc::SerializeSchema(*poTmpSchema, m_poMemoryPool);
+            if (status.ok())
             {
-                // The serialized schema is not UTF-8, which is required for Thrift
+                // The serialized schema is not UTF-8, which is required for
+                // Thrift
                 std::string schema_as_string = (*status)->ToString();
-                std::string schema_base64 = ::arrow::util::base64_encode(schema_as_string);
+                std::string schema_base64 =
+                    ::arrow::util::base64_encode(schema_as_string);
                 static const std::string kArrowSchemaKey = "ARROW:schema";
-                const_cast<arrow::KeyValueMetadata*>(m_poKeyValueMetadata.get())->Append(
-                  kArrowSchemaKey, schema_base64);
+                const_cast<arrow::KeyValueMetadata *>(
+                    m_poKeyValueMetadata.get())
+                    ->Append(kArrowSchemaKey, schema_base64);
             }
         }
     }
@@ -550,28 +561,30 @@ void OGRParquetWriterLayer::PerformStepsBeforeFinalFlushGroup()
 
 // Same as parquet::arrow::FileWriter::Open(), except we also
 // return KeyValueMetadata
-static
-arrow::Status Open(const ::arrow::Schema& schema, ::arrow::MemoryPool* pool,
-                        std::shared_ptr<::arrow::io::OutputStream> sink,
-                        std::shared_ptr<parquet::WriterProperties> properties,
-                        std::shared_ptr<parquet::ArrowWriterProperties> arrow_properties,
-                        std::unique_ptr<parquet::arrow::FileWriter>* writer,
-                        std::shared_ptr<const arrow::KeyValueMetadata>* outMetadata) {
+static arrow::Status
+Open(const ::arrow::Schema &schema, ::arrow::MemoryPool *pool,
+     std::shared_ptr<::arrow::io::OutputStream> sink,
+     std::shared_ptr<parquet::WriterProperties> properties,
+     std::shared_ptr<parquet::ArrowWriterProperties> arrow_properties,
+     std::unique_ptr<parquet::arrow::FileWriter> *writer,
+     std::shared_ptr<const arrow::KeyValueMetadata> *outMetadata)
+{
     std::shared_ptr<parquet::SchemaDescriptor> parquet_schema;
-    RETURN_NOT_OK(
-      parquet::arrow::ToParquetSchema(&schema, *properties, *arrow_properties, &parquet_schema));
+    RETURN_NOT_OK(parquet::arrow::ToParquetSchema(
+        &schema, *properties, *arrow_properties, &parquet_schema));
 
-    auto schema_node = std::static_pointer_cast<parquet::schema::GroupNode>(parquet_schema->schema_root());
+    auto schema_node = std::static_pointer_cast<parquet::schema::GroupNode>(
+        parquet_schema->schema_root());
 
-    auto metadata = schema.metadata() ? schema.metadata()->Copy() :
-                        std::make_shared<arrow::KeyValueMetadata>();
+    auto metadata = schema.metadata()
+                        ? schema.metadata()->Copy()
+                        : std::make_shared<arrow::KeyValueMetadata>();
     *outMetadata = metadata;
 
     std::unique_ptr<parquet::ParquetFileWriter> base_writer;
-    PARQUET_CATCH_NOT_OK(base_writer = parquet::ParquetFileWriter::Open(
-        std::move(sink), schema_node,
-        std::move(properties),
-        metadata));
+    PARQUET_CATCH_NOT_OK(
+        base_writer = parquet::ParquetFileWriter::Open(
+            std::move(sink), schema_node, std::move(properties), metadata));
 
     auto schema_ptr = std::make_shared<::arrow::Schema>(schema);
     return parquet::arrow::FileWriter::Make(
@@ -592,16 +605,18 @@ void OGRParquetWriterLayer::CreateSchema()
 /*                          CreateGeomField()                           */
 /************************************************************************/
 
-OGRErr OGRParquetWriterLayer::CreateGeomField( OGRGeomFieldDefn *poField,
-                                               int bApproxOK )
+OGRErr OGRParquetWriterLayer::CreateGeomField(OGRGeomFieldDefn *poField,
+                                              int bApproxOK)
 {
     OGRErr eErr = OGRArrowWriterLayer::CreateGeomField(poField, bApproxOK);
-    if( eErr == OGRERR_NONE && m_aeGeomEncoding.back() == OGRArrowGeomEncoding::WKB )
+    if (eErr == OGRERR_NONE &&
+        m_aeGeomEncoding.back() == OGRArrowGeomEncoding::WKB)
     {
         m_oWriterPropertiesBuilder.disable_statistics(
             parquet::schema::ColumnPath::FromDotString(
-                m_poFeatureDefn->GetGeomFieldDefn(
-                    m_poFeatureDefn->GetGeomFieldCount() - 1)->GetNameRef()));
+                m_poFeatureDefn
+                    ->GetGeomFieldDefn(m_poFeatureDefn->GetGeomFieldCount() - 1)
+                    ->GetNameRef()));
     }
     return eErr;
 }
@@ -612,9 +627,9 @@ OGRErr OGRParquetWriterLayer::CreateGeomField( OGRGeomFieldDefn *poField,
 
 void OGRParquetWriterLayer::CreateWriter()
 {
-    CPLAssert( m_poFileWriter == nullptr );
+    CPLAssert(m_poFileWriter == nullptr);
 
-    if( m_poSchema == nullptr )
+    if (m_poSchema == nullptr)
     {
         CreateSchema();
     }
@@ -623,12 +638,12 @@ void OGRParquetWriterLayer::CreateWriter()
         FinalizeSchema();
     }
 
-    auto arrowWriterProperties = parquet::ArrowWriterProperties::Builder().store_schema()->build();
+    auto arrowWriterProperties =
+        parquet::ArrowWriterProperties::Builder().store_schema()->build();
     CPL_IGNORE_RET_VAL(Open(*m_poSchema, m_poMemoryPool, m_poOutputStream,
-         m_oWriterPropertiesBuilder.build(),
-         arrowWriterProperties,
-         &m_poFileWriter,
-         &m_poKeyValueMetadata));
+                            m_oWriterPropertiesBuilder.build(),
+                            arrowWriterProperties, &m_poFileWriter,
+                            &m_poKeyValueMetadata));
 }
 
 /************************************************************************/
@@ -638,28 +653,28 @@ void OGRParquetWriterLayer::CreateWriter()
 bool OGRParquetWriterLayer::FlushGroup()
 {
     auto status = m_poFileWriter->NewRowGroup(m_apoBuilders[0]->length());
-    if( !status.ok() )
+    if (!status.ok())
     {
-        CPLError(CE_Failure, CPLE_AppDefined,
-                 "NewRowGroup() failed with %s", status.message().c_str());
+        CPLError(CE_Failure, CPLE_AppDefined, "NewRowGroup() failed with %s",
+                 status.message().c_str());
         m_apoBuilders.clear();
         return false;
     }
 
-    auto ret = WriteArrays([this](const std::shared_ptr<arrow::Field>& field,
-                                  const std::shared_ptr<arrow::Array>& array)
-    {
-        auto l_status = m_poFileWriter->WriteColumnChunk(*array);
-        if( !l_status.ok() )
+    auto ret = WriteArrays(
+        [this](const std::shared_ptr<arrow::Field> &field,
+               const std::shared_ptr<arrow::Array> &array)
         {
-            CPLError(CE_Failure, CPLE_AppDefined,
-                 "WriteColumnChunk() failed for field %s: %s",
-                 field->name().c_str(),
-                 l_status.message().c_str());
-            return false;
-        }
-        return true;
-    });
+            auto l_status = m_poFileWriter->WriteColumnChunk(*array);
+            if (!l_status.ok())
+            {
+                CPLError(CE_Failure, CPLE_AppDefined,
+                         "WriteColumnChunk() failed for field %s: %s",
+                         field->name().c_str(), l_status.message().c_str());
+                return false;
+            }
+            return true;
+        });
 
     m_apoBuilders.clear();
     return ret;
@@ -669,31 +684,31 @@ bool OGRParquetWriterLayer::FlushGroup()
 /*                     FixupGeometryBeforeWriting()                     */
 /************************************************************************/
 
-void OGRParquetWriterLayer::FixupGeometryBeforeWriting(OGRGeometry* poGeom)
+void OGRParquetWriterLayer::FixupGeometryBeforeWriting(OGRGeometry *poGeom)
 {
-    if( !m_bForceCounterClockwiseOrientation )
+    if (!m_bForceCounterClockwiseOrientation)
         return;
 
     const auto eFlattenType = wkbFlatten(poGeom->getGeometryType());
     // Polygon rings MUST follow the right-hand rule for orientation
     // (counterclockwise external rings, clockwise internal rings)
-    if( eFlattenType == wkbPolygon )
+    if (eFlattenType == wkbPolygon)
     {
         bool bFirstRing = true;
-        for( auto poRing: poGeom->toPolygon() )
+        for (auto poRing : poGeom->toPolygon())
         {
-            if( (bFirstRing && poRing->isClockwise()) ||
-                (!bFirstRing && !poRing->isClockwise()) )
+            if ((bFirstRing && poRing->isClockwise()) ||
+                (!bFirstRing && !poRing->isClockwise()))
             {
                 poRing->reverseWindingOrder();
             }
             bFirstRing = false;
         }
     }
-    else if( eFlattenType == wkbMultiPolygon ||
-             eFlattenType == wkbGeometryCollection )
+    else if (eFlattenType == wkbMultiPolygon ||
+             eFlattenType == wkbGeometryCollection)
     {
-        for( auto poSubGeom: poGeom->toGeometryCollection() )
+        for (auto poSubGeom : poGeom->toGeometryCollection())
         {
             FixupGeometryBeforeWriting(poSubGeom);
         }
