@@ -142,12 +142,13 @@ class OGRSQLiteBaseDataSource CPL_NON_FINAL : public GDALPamDataset
         nullptr; /* Set by the VFS layer when it opens the DB */
                  /* Must *NOT* be closed by the datasource explicitly. */
 
-    int OpenOrCreateDB(int flags, bool bRegisterOGR2SQLiteExtensions);
+    bool OpenOrCreateDB(int flags, bool bRegisterOGR2SQLiteExtensions,
+                        bool bLoadExtensions);
     bool SetSynchronous();
     bool SetCacheSize();
     void LoadExtensions();
 
-    void CloseDB();
+    bool CloseDB();
 
     std::map<CPLString, OGREnvelope> oMapSQLEnvelope{};
 
@@ -156,8 +157,8 @@ class OGRSQLiteBaseDataSource CPL_NON_FINAL : public GDALPamDataset
         m_osMapRelationships{};
 
     void *hSpatialiteCtxt = nullptr;
-    bool InitNewSpatialite();
-    void FinishNewSpatialite();
+    bool InitSpatialite();
+    void FinishSpatialite();
 
     int bUserTransactionActive = FALSE;
     int nSoftTransactionLevel = 0;
@@ -197,6 +198,8 @@ class OGRSQLiteBaseDataSource CPL_NON_FINAL : public GDALPamDataset
     GetLayerWithGetSpatialWhereByName(const char *pszName) = 0;
 
     virtual OGRErr AbortSQL() override;
+    bool SetQueryLoggerFunc(GDALQueryLoggerFunc pfnQueryLoggerFuncIn,
+                            void *poQueryLoggerArgIn) override;
 
     virtual OGRErr StartTransaction(int bForce = FALSE) override;
     virtual OGRErr CommitTransaction() override;
@@ -214,6 +217,18 @@ class OGRSQLiteBaseDataSource CPL_NON_FINAL : public GDALPamDataset
                        int nRowsExpected);
 
     void LoadRelationshipsFromForeignKeys() const;
+
+    // sqlite3_prepare_v2 error logging wrapper
+    int
+    prepareSql(sqlite3 *db,           /* Database handle */
+               const char *zSql,      /* SQL statement, UTF-8 encoded */
+               int nByte,             /* Maximum length of zSql in bytes. */
+               sqlite3_stmt **ppStmt, /* OUT: Statement handle */
+               const char **pzTail /* OUT: Pointer to unused portion of zSql */
+    );
+
+    GDALQueryLoggerFunc pfnQueryLoggerFunc = nullptr;
+    void *poQueryLoggerArg = nullptr;
 };
 
 /************************************************************************/
