@@ -49,53 +49,6 @@
 #include "ogr_spatialref.h"
 #include "sqlite3.h"
 
-/************************************************************************/
-/*                       OGRSQLiteExecuteSQLLayer                       */
-/************************************************************************/
-
-class OGRSQLiteExecuteSQLLayer final : public OGRSQLiteSelectLayer
-{
-    char *pszTmpDBName;
-
-  public:
-    OGRSQLiteExecuteSQLLayer(char *pszTmpDBName, OGRSQLiteDataSource *poDS,
-                             const CPLString &osSQL, sqlite3_stmt *hStmt,
-                             bool bUseStatementForGetNextFeature,
-                             bool bEmptyLayer);
-    virtual ~OGRSQLiteExecuteSQLLayer();
-};
-
-/************************************************************************/
-/*                         OGRSQLiteExecuteSQLLayer()                   */
-/************************************************************************/
-
-OGRSQLiteExecuteSQLLayer::OGRSQLiteExecuteSQLLayer(
-    char *pszTmpDBNameIn, OGRSQLiteDataSource *poDSIn, const CPLString &osSQL,
-    sqlite3_stmt *hStmtIn, bool bUseStatementForGetNextFeature,
-    bool bEmptyLayer)
-    : OGRSQLiteSelectLayer(poDSIn, osSQL, hStmtIn,
-                           bUseStatementForGetNextFeature, bEmptyLayer, true),
-      pszTmpDBName(pszTmpDBNameIn)
-{
-}
-
-/************************************************************************/
-/*                        ~OGRSQLiteExecuteSQLLayer()                   */
-/************************************************************************/
-
-OGRSQLiteExecuteSQLLayer::~OGRSQLiteExecuteSQLLayer()
-{
-    // This is a bit peculiar: we must "finalize" the OGRLayer, since
-    // it has objects that depend on the datasource, that we are just
-    // going to destroy afterwards. The issue here is that we destroy
-    // our own datasource,
-    Finalize();
-
-    delete m_poDS;
-    VSIUnlink(pszTmpDBName);
-    CPLFree(pszTmpDBName);
-}
-
 /************************************************************************/
 /*                       OGR2SQLITEExtractUnquotedString()              */
 /************************************************************************/
@@ -104,34 +57,77 @@ static CPLString OGR2SQLITEExtractUnquotedString(const char **ppszSQLCommand)
 {
     CPLString osRet;
     const char *pszSQLCommand = *ppszSQLCommand;
-    char chQuoteChar = 0;
-
     if (*pszSQLCommand == '"' || *pszSQLCommand == '\'')
     {
-        chQuoteChar = *pszSQLCommand;
+        const char chQuoteChar = *pszSQLCommand;
         pszSQLCommand++;
+
+        while (*pszSQLCommand != '\0')
+        {
+            if (*pszSQLCommand == chQuoteChar &&
+                pszSQLCommand[1] == chQuoteChar)
+            {
+                pszSQLCommand++;
+                osRet += chQuoteChar;
+            }
+            else if (*pszSQLCommand == chQuoteChar)
+            {
+                pszSQLCommand++;
+                break;
+            }
+            else
+                osRet += *pszSQLCommand;
+
+            pszSQLCommand++;
+        }
     }
-
-    while (*pszSQLCommand != '\0')
+    else
     {
-        if (*pszSQLCommand == chQuoteChar && pszSQLCommand[1] == chQuoteChar)
+        bool bNotATableName = false;
+        char chQuoteChar = 0;
+        int nParenthesisLevel = 0;
+        while (*pszSQLCommand != '\0')
         {
-            pszSQLCommand++;
-            osRet += chQuoteChar;
-        }
-        else if (*pszSQLCommand == chQuoteChar)
-        {
-            pszSQLCommand++;
-            break;
-        }
-        else if (chQuoteChar == '\0' &&
-                 (isspace((int)*pszSQLCommand) || *pszSQLCommand == '.' ||
-                  *pszSQLCommand == ')' || *pszSQLCommand == ','))
-            break;
-        else
-            osRet += *pszSQLCommand;
+            if (*pszSQLCommand == chQuoteChar &&
+                pszSQLCommand[1] == chQuoteChar)
+            {
+                osRet += *pszSQLCommand;
+                pszSQLCommand++;
+            }
+            else if (*pszSQLCommand == chQuoteChar)
+            {
+                chQuoteChar = 0;
+            }
+            else if (chQuoteChar == 0)
+            {
+                if (*pszSQLCommand == '(')
+                {
+                    bNotATableName = true;
+                    nParenthesisLevel++;
+                }
+                else if (*pszSQLCommand == ')')
+                {
+                    nParenthesisLevel--;
+                    if (nParenthesisLevel < 0)
+                        break;
+                }
+                else if (*pszSQLCommand == '"' || *pszSQLCommand == '\'')
+                {
+                    chQuoteChar = *pszSQLCommand;
+                }
+                else if (nParenthesisLevel == 0 &&
+                         (isspace((int)*pszSQLCommand) ||
+                          *pszSQLCommand == '.' || *pszSQLCommand == ','))
+                {
+                    break;
+                }
+            }
 
-        pszSQLCommand++;
+            osRet += *pszSQLCommand;
+            pszSQLCommand++;
+        }
+        if (bNotATableName)
+            osRet.clear();
     }
 
     *ppszSQLCommand = pszSQLCommand;
@@ -460,6 +456,83 @@ static void OGR2SQLITEGetPotentialLayerNames(
         &pszSQLCommand, oSetLayers, oSetSpatialIndex, osModifiedSQL, nNum);
 }
 
+/************************************************************************/
+/*                   OGRSQLiteGetReferencedLayers()                     */
+/************************************************************************/
+
+std::set<LayerDesc> OGRSQLiteGetReferencedLayers(const char *pszStatement)
+{
+    /* -------------------------------------------------------------------- */
+    /*      Analyze the statement to determine which tables will be used.   */
+    /* -------------------------------------------------------------------- */
+    std::set<LayerDesc> oSetLayers;
+    std::set<CPLString> oSetSpatialIndex;
+    CPLString osModifiedSQL;
+    OGR2SQLITEGetPotentialLayerNames(pszStatement, oSetLayers, oSetSpatialIndex,
+                                     osModifiedSQL);
+
+    return oSetLayers;
+}
+
+#ifndef HAVE_SQLITE3EXT_H
+OGRLayer *OGRSQLiteExecuteSQL(GDALDataset *, const char *, OGRGeometry *,
+                              const char *)
+{
+    CPLError(CE_Failure, CPLE_NotSupported,
+             "SQL SQLite dialect not supported due to GDAL being built "
+             "without sqlite3ext.h header");
+    return nullptr;
+}
+
+#else
+
+/************************************************************************/
+/*                       OGRSQLiteExecuteSQLLayer                       */
+/************************************************************************/
+
+class OGRSQLiteExecuteSQLLayer final : public OGRSQLiteSelectLayer
+{
+    char *pszTmpDBName;
+
+  public:
+    OGRSQLiteExecuteSQLLayer(char *pszTmpDBName, OGRSQLiteDataSource *poDS,
+                             const CPLString &osSQL, sqlite3_stmt *hStmt,
+                             bool bUseStatementForGetNextFeature,
+                             bool bEmptyLayer);
+    virtual ~OGRSQLiteExecuteSQLLayer();
+};
+
+/************************************************************************/
+/*                         OGRSQLiteExecuteSQLLayer()                   */
+/************************************************************************/
+
+OGRSQLiteExecuteSQLLayer::OGRSQLiteExecuteSQLLayer(
+    char *pszTmpDBNameIn, OGRSQLiteDataSource *poDSIn, const CPLString &osSQL,
+    sqlite3_stmt *hStmtIn, bool bUseStatementForGetNextFeature,
+    bool bEmptyLayer)
+    : OGRSQLiteSelectLayer(poDSIn, osSQL, hStmtIn,
+                           bUseStatementForGetNextFeature, bEmptyLayer, true),
+      pszTmpDBName(pszTmpDBNameIn)
+{
+}
+
+/************************************************************************/
+/*                        ~OGRSQLiteExecuteSQLLayer()                   */
+/************************************************************************/
+
+OGRSQLiteExecuteSQLLayer::~OGRSQLiteExecuteSQLLayer()
+{
+    // This is a bit peculiar: we must "finalize" the OGRLayer, since
+    // it has objects that depend on the datasource, that we are just
+    // going to destroy afterwards. The issue here is that we destroy
+    // our own datasource,
+    Finalize();
+
+    delete m_poDS;
+    VSIUnlink(pszTmpDBName);
+    CPLFree(pszTmpDBName);
+}
+
 /************************************************************************/
 /*               OGR2SQLITE_IgnoreAllFieldsExceptGeometry()             */
 /************************************************************************/
@@ -515,7 +588,7 @@ static int OGR2SQLITEDealWithSpatialColumn(
     CPLString osIdxNameEscaped(SQLEscapeName(osIdxNameRaw));
 
     /* Make sure that the SRS is injected in spatial_ref_sys */
-    OGRSpatialReference *poSRS = poGeomField->GetSpatialRef();
+    const OGRSpatialReference *poSRS = poGeomField->GetSpatialRef();
     if (iGeomCol == 0 && poSRS == nullptr)
         poSRS = poLayer->GetSpatialRef();
     int nSRSId = poSQLiteDS->GetUndefinedSRID();
@@ -724,15 +797,6 @@ OGRLayer *OGRSQLiteExecuteSQL(GDALDataset *poDS, const char *pszStatement,
     OGRSQLiteDataSource *poSQLiteDS = nullptr;
     bool bSpatialiteDB = false;
 
-    CPLString osOldVal;
-    const char *pszOldVal =
-        CPLGetConfigOption("OGR_SQLITE_STATIC_VIRTUAL_OGR", nullptr);
-    if (pszOldVal != nullptr)
-    {
-        osOldVal = pszOldVal;
-        pszOldVal = osOldVal.c_str();
-    }
-
     /* -------------------------------------------------------------------- */
     /*      Create in-memory sqlite/spatialite DB                           */
     /* -------------------------------------------------------------------- */
@@ -797,11 +861,10 @@ OGRLayer *OGRSQLiteExecuteSQL(GDALDataset *poDS, const char *pszStatement,
                                         nEmptyDBSize, TRUE));
 
         poSQLiteDS = new OGRSQLiteDataSource();
-        CPLSetThreadLocalConfigOption("OGR_SQLITE_STATIC_VIRTUAL_OGR", "NO");
         GDALOpenInfo oOpenInfo(pszTmpDBName, GDAL_OF_VECTOR | GDAL_OF_UPDATE);
+        CPLConfigOptionSetter oSetter("OGR_SQLITE_STATIC_VIRTUAL_OGR", "NO",
+                                      false);
         const int nRet = poSQLiteDS->Open(&oOpenInfo);
-        CPLSetThreadLocalConfigOption("OGR_SQLITE_STATIC_VIRTUAL_OGR",
-                                      pszOldVal);
         if (!nRet)
         {
             /* should not happen really ! */
@@ -816,14 +879,16 @@ OGRLayer *OGRSQLiteExecuteSQL(GDALDataset *poDS, const char *pszStatement,
     /* No caching version */
     poSQLiteDS = new OGRSQLiteDataSource();
     char **papszOptions = CSLAddString(NULL, "SPATIALITE=YES");
-    CPLSetThreadLocalConfigOption("OGR_SQLITE_STATIC_VIRTUAL_OGR", "NO");
-    const int nRet = poSQLiteDS->Create(pszTmpDBName, papszOptions);
-    CPLSetThreadLocalConfigOption("OGR_SQLITE_STATIC_VIRTUAL_OGR", pszOldVal);
-    CSLDestroy(papszOptions);
-    papszOptions = NULL;
-    if (nRet)
     {
-        bSpatialiteDB = true;
+        CPLConfigOptionSetter oSetter("OGR_SQLITE_STATIC_VIRTUAL_OGR", "NO",
+                                      false);
+        const int nRet = poSQLiteDS->Create(pszTmpDBName, papszOptions);
+        CSLDestroy(papszOptions);
+        papszOptions = nullptr;
+        if (nRet)
+        {
+            bSpatialiteDB = true;
+        }
     }
 #endif
 
@@ -838,10 +903,9 @@ OGRLayer *OGRSQLiteExecuteSQL(GDALDataset *poDS, const char *pszStatement,
 
         // cppcheck-suppress redundantAssignment
         poSQLiteDS = new OGRSQLiteDataSource();
-        CPLSetThreadLocalConfigOption("OGR_SQLITE_STATIC_VIRTUAL_OGR", "NO");
+        CPLConfigOptionSetter oSetter("OGR_SQLITE_STATIC_VIRTUAL_OGR", "NO",
+                                      false);
         const int nRet = poSQLiteDS->Create(pszTmpDBName, nullptr);
-        CPLSetThreadLocalConfigOption("OGR_SQLITE_STATIC_VIRTUAL_OGR",
-                                      pszOldVal);
         if (!nRet)
         {
             delete poSQLiteDS;
@@ -1051,20 +1115,4 @@ OGRLayer *OGRSQLiteExecuteSQL(GDALDataset *poDS, const char *pszStatement,
     return poLayer;
 }
 
-/************************************************************************/
-/*                   OGRSQLiteGetReferencedLayers()                     */
-/************************************************************************/
-
-std::set<LayerDesc> OGRSQLiteGetReferencedLayers(const char *pszStatement)
-{
-    /* -------------------------------------------------------------------- */
-    /*      Analysze the statement to determine which tables will be used.  */
-    /* -------------------------------------------------------------------- */
-    std::set<LayerDesc> oSetLayers;
-    std::set<CPLString> oSetSpatialIndex;
-    CPLString osModifiedSQL;
-    OGR2SQLITEGetPotentialLayerNames(pszStatement, oSetLayers, oSetSpatialIndex,
-                                     osModifiedSQL);
-
-    return oSetLayers;
-}
+#endif  // HAVE_SQLITE3EXT_H
