@@ -81,14 +81,13 @@ OGRWFSLayer::OGRWFSLayer(OGRWFSDataSource *poDSIn, OGRSpatialReference *poSRSIn,
                          const char *pszNSIn, const char *pszNSValIn)
     : poDS(poDSIn), poFeatureDefn(nullptr), bGotApproximateLayerDefn(false),
       poGMLFeatureClass(nullptr),
-      bAxisOrderAlreadyInverted(bAxisOrderAlreadyInvertedIn), poSRS(poSRSIn),
+      bAxisOrderAlreadyInverted(bAxisOrderAlreadyInvertedIn), m_poSRS(poSRSIn),
       pszBaseURL(CPLStrdup(pszBaseURLIn)), pszName(CPLStrdup(pszNameIn)),
       pszNS(pszNSIn ? CPLStrdup(pszNSIn) : nullptr),
       pszNSVal(pszNSValIn ? CPLStrdup(pszNSValIn) : nullptr),
       bStreamingDS(false), poBaseDS(nullptr), poBaseLayer(nullptr),
       bHasFetched(false), bReloadNeeded(false), eGeomType(wkbUnknown),
-      nFeatures(-1), bCountFeaturesInGetNextFeature(false), dfMinX(0.0),
-      dfMinY(0.0), dfMaxX(0.0), dfMaxY(0.0), bHasExtents(false),
+      nFeatures(-1), bCountFeaturesInGetNextFeature(false),
       poFetchedFilterGeom(nullptr), nExpectedInserts(0), bInTransaction(false),
       bUseFeatureIdAtLayerLevel(false), bPagingActive(false),
       nPagingStartIndex(0), nFeatureRead(0), nFeatureCountRequested(0),
@@ -104,10 +103,10 @@ OGRWFSLayer::OGRWFSLayer(OGRWFSDataSource *poDSIn, OGRSpatialReference *poSRSIn,
 OGRWFSLayer *OGRWFSLayer::Clone()
 {
     OGRWFSLayer *poDupLayer =
-        new OGRWFSLayer(poDS, poSRS, bAxisOrderAlreadyInverted, pszBaseURL,
+        new OGRWFSLayer(poDS, m_poSRS, bAxisOrderAlreadyInverted, pszBaseURL,
                         pszName, pszNS, pszNSVal);
-    if (poSRS)
-        poSRS->Reference();
+    if (m_poSRS)
+        m_poSRS->Reference();
     poDupLayer->poFeatureDefn = GetLayerDefn()->Clone();
     poDupLayer->poFeatureDefn->Reference();
     poDupLayer->bGotApproximateLayerDefn = bGotApproximateLayerDefn;
@@ -134,8 +133,8 @@ OGRWFSLayer::~OGRWFSLayer()
     if (bInTransaction)
         OGRWFSLayer::CommitTransaction();
 
-    if (poSRS != nullptr)
-        poSRS->Release();
+    if (m_poSRS != nullptr)
+        m_poSRS->Release();
 
     if (poFeatureDefn != nullptr)
         poFeatureDefn->Release();
@@ -156,6 +155,61 @@ OGRWFSLayer::~OGRWFSLayer()
     CPLFree(pszRequiredOutputFormat);
 }
 
+/************************************************************************/
+/*                          SetActiveSRS()                              */
+/************************************************************************/
+
+OGRErr OGRWFSLayer::SetActiveSRS(int /*iGeomField*/,
+                                 const OGRSpatialReference *poSRS)
+{
+    if (poSRS == nullptr)
+        return OGRERR_FAILURE;
+    const char *const apszOptions[] = {
+        "IGNORE_DATA_AXIS_TO_SRS_AXIS_MAPPING=YES", nullptr};
+    size_t i = 0;
+    for (const auto &poSupportedSRS : m_apoSupportedCRSList)
+    {
+        if (poSupportedSRS->IsSame(poSRS, apszOptions))
+        {
+            m_osSRSName = m_aosSupportedCRSList[i];
+            if (m_poSRS)
+                m_poSRS->Release();
+            m_poSRS = poSRS->Clone();
+            m_poSRS->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
+            if (poFeatureDefn)
+            {
+                auto poGeomFieldDefn = poFeatureDefn->GetGeomFieldDefn(0);
+                if (poGeomFieldDefn)
+                {
+                    poGeomFieldDefn->SetSpatialRef(m_poSRS);
+                }
+            }
+            m_oExtents = OGREnvelope();
+            if (m_oWGS84Extents.IsInit())
+            {
+                OGRSpatialReference oWGS84;
+                oWGS84.SetWellKnownGeogCS("WGS84");
+                oWGS84.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
+                auto poCT = std::unique_ptr<OGRCoordinateTransformation>(
+                    OGRCreateCoordinateTransformation(&oWGS84, m_poSRS));
+                if (poCT)
+                {
+                    poCT->TransformBounds(
+                        m_oWGS84Extents.MinX, m_oWGS84Extents.MinY,
+                        m_oWGS84Extents.MaxX, m_oWGS84Extents.MaxY,
+                        &m_oExtents.MinX, &m_oExtents.MinY, &m_oExtents.MaxX,
+                        &m_oExtents.MaxY, 20);
+                }
+            }
+            SetSpatialFilter(nullptr);
+            ResetReading();
+            return OGRERR_NONE;
+        }
+        ++i;
+    }
+    return OGRERR_FAILURE;
+}
+
 /************************************************************************/
 /*                    GetDescribeFeatureTypeURL()                       */
 /************************************************************************/
@@ -306,7 +360,7 @@ OGRWFSLayer::BuildLayerDefnFromFeatureClass(GMLFeatureClass *poClass)
         poFDefn->SetGeomType(
             (OGRwkbGeometryType)poGMLFeatureClass->GetGeometryProperty(0)
                 ->GetType());
-        poFDefn->GetGeomFieldDefn(0)->SetSpatialRef(poSRS);
+        poFDefn->GetGeomFieldDefn(0)->SetSpatialRef(m_poSRS);
     }
 
     /* -------------------------------------------------------------------- */
@@ -375,6 +429,9 @@ CPLString OGRWFSLayer::MakeGetFeatureURL(int nRequestMaxFeatures,
         osURL = CPLURLAddKVP(osURL, "TYPENAMES", WFS_EscapeURL(pszName));
     else
         osURL = CPLURLAddKVP(osURL, "TYPENAME", WFS_EscapeURL(pszName));
+    if (!m_osSRSName.empty())
+        osURL =
+            CPLURLAddKVP(osURL, "SRSNAME", WFS_EscapeURL(m_osSRSName.c_str()));
     if (pszRequiredOutputFormat)
         osURL = CPLURLAddKVP(osURL, "OUTPUTFORMAT",
                              WFS_EscapeURL(pszRequiredOutputFormat));
@@ -1099,7 +1156,7 @@ OGRFeatureDefn *OGRWFSLayer::BuildLayerDefn(OGRFeatureDefn *poSrcFDefn)
     bool bUnsetWidthPrecision = false;
 
     poFeatureDefn = new OGRFeatureDefn(pszName);
-    poFeatureDefn->GetGeomFieldDefn(0)->SetSpatialRef(poSRS);
+    poFeatureDefn->GetGeomFieldDefn(0)->SetSpatialRef(m_poSRS);
     poFeatureDefn->Reference();
 
     GDALDataset *l_poDS = nullptr;
@@ -1168,7 +1225,8 @@ OGRFeatureDefn *OGRWFSLayer::BuildLayerDefn(OGRFeatureDefn *poSrcFDefn)
 void OGRWFSLayer::ResetReading()
 
 {
-    GetLayerDefn();
+    if (poFeatureDefn == nullptr)
+        return;
     if (bPagingActive)
         bReloadNeeded = true;
     nPagingStartIndex = 0;
@@ -1326,8 +1384,8 @@ OGRFeature *OGRWFSLayer::GetNextFeature()
             poGeom->swapXY();
         }
 
-        if (poGeom && poSRS)
-            poGeom->assignSpatialReference(poSRS);
+        if (poGeom && m_poSRS)
+            poGeom->assignSpatialReference(m_poSRS);
         delete poSrcFeature;
         return poNewFeature;
     }
@@ -1466,7 +1524,7 @@ int OGRWFSLayer::TestCapability(const char *pszCap)
 
     else if (EQUAL(pszCap, OLCFastGetExtent))
     {
-        if (bHasExtents)
+        if (m_oExtents.IsInit())
             return TRUE;
 
         return poBaseLayer != nullptr && poBaseLayer->TestCapability(pszCap);
@@ -1657,7 +1715,7 @@ int OGRWFSLayer::CanRunGetFeatureCountAndGetExtentTogether()
     /* In some cases, we can evaluate the result of GetFeatureCount() */
     /* and GetExtent() with the same data */
     CPLString osRequestURL = MakeGetFeatureURL(0, FALSE);
-    return (!bHasExtents && nFeatures < 0 &&
+    return (!m_oExtents.IsInit() && nFeatures < 0 &&
             osRequestURL.ifind("FILTER") == std::string::npos &&
             osRequestURL.ifind("MAXFEATURES") == std::string::npos &&
             osRequestURL.ifind("COUNT") == std::string::npos &&
@@ -1714,17 +1772,29 @@ GIntBig OGRWFSLayer::GetFeatureCount(int bForce)
 }
 
 /************************************************************************/
-/*                              SetExtent()                             */
+/*                              SetExtents()                            */
 /************************************************************************/
 
 void OGRWFSLayer::SetExtents(double dfMinXIn, double dfMinYIn, double dfMaxXIn,
                              double dfMaxYIn)
 {
-    dfMinX = dfMinXIn;
-    dfMinY = dfMinYIn;
-    dfMaxX = dfMaxXIn;
-    dfMaxY = dfMaxYIn;
-    bHasExtents = true;
+    m_oExtents.MinX = dfMinXIn;
+    m_oExtents.MinY = dfMinYIn;
+    m_oExtents.MaxX = dfMaxXIn;
+    m_oExtents.MaxY = dfMaxYIn;
+}
+
+/************************************************************************/
+/*                            SetWGS84Extents()                         */
+/************************************************************************/
+
+void OGRWFSLayer::SetWGS84Extents(double dfMinXIn, double dfMinYIn,
+                                  double dfMaxXIn, double dfMaxYIn)
+{
+    m_oWGS84Extents.MinX = dfMinXIn;
+    m_oWGS84Extents.MinY = dfMinYIn;
+    m_oWGS84Extents.MaxX = dfMaxXIn;
+    m_oWGS84Extents.MaxY = dfMaxYIn;
 }
 
 /************************************************************************/
@@ -1733,12 +1803,9 @@ void OGRWFSLayer::SetExtents(double dfMinXIn, double dfMinYIn, double dfMaxXIn,
 
 OGRErr OGRWFSLayer::GetExtent(OGREnvelope *psExtent, int bForce)
 {
-    if (bHasExtents)
+    if (m_oExtents.IsInit())
     {
-        psExtent->MinX = dfMinX;
-        psExtent->MinY = dfMinY;
-        psExtent->MaxX = dfMaxX;
-        psExtent->MaxY = dfMaxY;
+        *psExtent = m_oExtents;
         return OGRERR_NONE;
     }
 
@@ -1771,11 +1838,7 @@ OGRErr OGRWFSLayer::GetExtent(OGREnvelope *psExtent, int bForce)
     {
         if (eErr == OGRERR_NONE)
         {
-            dfMinX = psExtent->MinX;
-            dfMinY = psExtent->MinY;
-            dfMaxX = psExtent->MaxX;
-            dfMaxY = psExtent->MaxY;
-            bHasExtents = true;
+            m_oExtents = *psExtent;
         }
         else
         {
@@ -1900,7 +1963,7 @@ OGRErr OGRWFSLayer::ICreateFeature(OGRFeature *poFeature)
             if (poGeom != nullptr && !osGeometryColumnName.empty())
             {
                 if (poGeom->getSpatialReference() == nullptr)
-                    poGeom->assignSpatialReference(poSRS);
+                    poGeom->assignSpatialReference(m_poSRS);
                 char *pszGML = nullptr;
                 if (strcmp(poDS->GetVersion(), "1.1.0") == 0 ||
                     atoi(poDS->GetVersion()) >= 2)
@@ -2103,7 +2166,7 @@ OGRErr OGRWFSLayer::ICreateFeature(OGRFeature *poFeature)
     /* Invalidate layer */
     bReloadNeeded = true;
     nFeatures = -1;
-    bHasExtents = false;
+    m_oExtents = OGREnvelope();
 
     return OGRERR_NONE;
 }
@@ -2168,7 +2231,7 @@ OGRErr OGRWFSLayer::ISetFeature(OGRFeature *poFeature)
         if (poGeom != nullptr)
         {
             if (poGeom->getSpatialReference() == nullptr)
-                poGeom->assignSpatialReference(poSRS);
+                poGeom->assignSpatialReference(m_poSRS);
             char *pszGML = nullptr;
             if (strcmp(poDS->GetVersion(), "1.1.0") == 0 ||
                 atoi(poDS->GetVersion()) >= 2)
@@ -2305,7 +2368,7 @@ OGRErr OGRWFSLayer::ISetFeature(OGRFeature *poFeature)
     /* Invalidate layer */
     bReloadNeeded = true;
     nFeatures = -1;
-    bHasExtents = false;
+    m_oExtents = OGREnvelope();
 
     return OGRERR_NONE;
 }
@@ -2451,7 +2514,7 @@ OGRErr OGRWFSLayer::DeleteFromFilter(CPLString osOGCFilter)
     /* Invalidate layer */
     bReloadNeeded = true;
     nFeatures = -1;
-    bHasExtents = false;
+    m_oExtents = OGREnvelope();
 
     return OGRERR_NONE;
 }
