@@ -273,6 +273,7 @@ class OGRXLSXDataSource final : public GDALDataset
   public:
     OGRXLSXDataSource();
     virtual ~OGRXLSXDataSource();
+    CPLErr Close() override;
 
     int Open(const char *pszFilename, const char *pszPrefixedFilename,
              VSILFILE *fpWorkbook, VSILFILE *fpWorkbookRels,
@@ -290,7 +291,7 @@ class OGRXLSXDataSource final : public GDALDataset
                                    char **papszOptions) override;
     virtual OGRErr DeleteLayer(int iLayer) override;
 
-    virtual void FlushCache(bool bAtClosing) override;
+    virtual CPLErr FlushCache(bool bAtClosing) override;
 
     void startElementCbk(const char *pszName, const char **ppszAttr);
     void endElementCbk(const char *pszName);
