@@ -676,6 +676,13 @@ void CPLJSONObject::Add(const std::string &osName, const CPLJSONObject &oValue)
     std::string objectName;
     if (m_osKey == INVALID_OBJ_KEY)
         m_osKey.clear();
+    if (osName.empty())
+    {
+        json_object_object_add(
+            TO_JSONOBJ(GetInternalHandle()), "",
+            json_object_get(TO_JSONOBJ(oValue.GetInternalHandle())));
+        return;
+    }
     CPLJSONObject object = GetObjectByPath(osName, objectName);
     if (object.IsValid() && json_object_get_type(TO_JSONOBJ(
                                 object.m_poJsonObject)) == json_type_object)
@@ -1456,3 +1463,45 @@ const CPLJSONObject CPLJSONArray::operator[](int nIndex) const
         CPLSPrintf("id:%d", nIndex),
         json_object_array_get_idx(TO_JSONOBJ(m_poJsonObject), nIndex));
 }
+
+/************************************************************************/
+/*                      CPLParseKeyValueJson()                          */
+/************************************************************************/
+
+/** Return a string list of key/value pairs extracted from a JSON doc.
+
+    We are expecting simple documents with key:value pairs, like the
+    following with no hierarchy or complex structure.
+    \verbatim
+    {
+      "Code" : "Success",
+      "LastUpdated" : "2017-07-03T16:20:17Z",
+      "Type" : "AWS-HMAC",
+      "AccessKeyId" : "bla",
+      "SecretAccessKey" : "bla",
+      "Token" : "bla",
+      "Expiration" : "2017-07-03T22:42:58Z"
+    }
+    \endverbatim
+    @since GDAL 3.7
+ */
+CPLStringList CPLParseKeyValueJson(const char *pszJson)
+{
+    CPLJSONDocument oDoc;
+    CPLStringList oNameValue;
+    if (pszJson != nullptr && oDoc.LoadMemory(pszJson))
+    {
+        for (const auto &obj : oDoc.GetRoot().GetChildren())
+        {
+            const auto eType = obj.GetType();
+            if (eType == CPLJSONObject::Type::String ||
+                eType == CPLJSONObject::Type::Integer ||
+                eType == CPLJSONObject::Type::Double)
+            {
+                oNameValue.SetNameValue(obj.GetName().c_str(),
+                                        obj.ToString().c_str());
+            }
+        }
+    }
+    return oNameValue;
+}
