@@ -26,6 +26,10 @@ for f in $FILES; do
 	# echo "Checking $f"
 	case "$f" in
 
+	*alg/internal_libqhull/*)
+	  continue
+	  ;;
+
 	*frmts/zlib/contrib/infback9/*)
 	  continue
 	  ;;
@@ -42,6 +46,14 @@ for f in $FILES; do
 	  continue
 	  ;;
 
+	*ogr/ogrsf_frmts/flatgeobuf/flatbuffers/*)
+	  continue
+	  ;;
+
+	*ogr/ogrsf_frmts/geojson/libjson*)
+	  continue
+	  ;;
+
 	*swig/*)
 	  continue
 	  ;;
